import { MarkersMixin } from "../mixins/marker-renderer";
import { PolylinesMixin } from "../mixins/polyline-renderer";
import { ExtendBoundsMixin } from "../mixins/extend-bounds";
import { TraceMixin } from "../mixins/trace";
import { StyleMixin } from "../mixins/styles";
import { Color } from "ht-utility";
import { MapItemsMixin } from "../mixins/map-items";
import { DataObservableMixin } from "../mixins/data-observable";
var ActionsExpectedPolyline = /** @class */ (function () {
    function ActionsExpectedPolyline(mapInstance) {
        this.mapInstance = mapInstance;
        this.lineSymbol = {
            path: 'M 0,-1 0,1',
            strokeOpacity: 1,
            scale: 4
        };
        this.styleFunct = {
            get: function (type) {
                switch (type) {
                    case "google":
                        {
                            return {
                                default: {
                                    strokeColor: Color.grey3,
                                    strokeOpacity: 0,
                                    strokeWeight: 5,
                                    icons: [{
                                            icon: {
                                                path: 'M 0,-1 0,0',
                                                strokeOpacity: 1,
                                                scale: 4
                                            },
                                            offset: '0',
                                            repeat: '13px'
                                        }],
                                },
                                highlight: {
                                    strokeColor: Color.grey2,
                                    strokeOpacity: 1,
                                    strokeWeight: 5
                                },
                                fade: {
                                    strokeColor: Color.grey2,
                                    strokeOpacity: 0.2,
                                    strokeWeight: 2
                                }
                            };
                        }
                        ;
                    case "leaflet": {
                        return {
                            default: {
                                weight: 5,
                                color: Color.grey4,
                                opacity: 1,
                                dashArray: "7 10"
                            },
                            highlight: {
                                weight: 5,
                                color: Color.grey2,
                                opacity: 1
                            },
                            fade: {
                                weight: 2,
                                color: Color.grey2,
                                opacity: 0.2
                            }
                        };
                    }
                }
            },
        };
        this.name = "action expected polyline";
    }
    ActionsExpectedPolyline.prototype.isValidMapItems = function (data) {
        return !!this.getEncodedPath(data);
    };
    ActionsExpectedPolyline.prototype.getEncodedPath = function (data) {
        return data.expected_route;
    };
    ;
    ActionsExpectedPolyline.prototype.getPosition = function () {
        return { lat: 0, lng: 0 };
    };
    return ActionsExpectedPolyline;
}());
export { ActionsExpectedPolyline };
export var ActionsExpectedPolylineTrace = DataObservableMixin(TraceMixin(ExtendBoundsMixin(PolylinesMixin(MarkersMixin(StyleMixin(MapItemsMixin(ActionsExpectedPolyline)))))));
//# sourceMappingURL=actions-expected-polyline.js.map