(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('ht-google-maps-wrapper'), require('ht-leaflet-wrapper'), require('rxjs/operators'), require('rxjs'), require('ht-utility'), require('time-aware-polyline'), require('underscore'), require('ht-data')) :
    typeof define === 'function' && define.amd ? define(['exports', 'ht-google-maps-wrapper', 'ht-leaflet-wrapper', 'rxjs/operators', 'rxjs', 'ht-utility', 'time-aware-polyline', 'underscore', 'ht-data'], factory) :
    (factory((global.htMaps = {}),global.htGoogleMapsWrapper,global.htLeafletWrapper,global.rxjs.operators,global.rxjs,global.htUtility,global.timeAwarePolyline,global._,global.htData));
}(this, (function (exports,htGoogleMapsWrapper,htLeafletWrapper,operators,rxjs,htUtility,timeAwarePolyline,_,htData) { 'use strict';

    var mapTypeService = (function () {
        var instance;
        var currentmapType;
        function getMapType(mapType) {
            switch (mapType) {
                case 'leaflet': {
                    return new htLeafletWrapper.LeafletMapUtilsClass();
                }
                case 'google': {
                    return new htGoogleMapsWrapper.GoogleMapUtilsClass();
                }
                default: {
                    return new htGoogleMapsWrapper.GoogleMapUtilsClass();
                }
            }
        }
        return {
            getInstance: function (mapType) {
                if (!instance || !currentmapType) {
                    currentmapType = mapType;
                    instance = getMapType(mapType);
                }
                return instance;
            }
        };
    })();

    var __assign = (undefined && undefined.__assign) || Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    var MapInstance = /** @class */ (function () {
        function MapInstance() {
            var _this = this;
            // mapUtils: MapUtils = null;
            this.map = null;
            this.map$ = new rxjs.ReplaySubject();
            this.clusters = [];
            this.poppers = [];
            this.itemsSet = [];
            // mapType: HtMapType;
            this.leafletSetBoundsOptions = {
                animate: true,
                duration: 0.3
            };
            this.googleSetBoundsOptions = {};
            this.map$.subscribe(function (map) {
                _this.map = map;
            });
        }
        Object.defineProperty(MapInstance.prototype, "mapUtils", {
            get: function () {
                return mapTypeService.getInstance();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(MapInstance.prototype, "mapType", {
            get: function () {
                return mapTypeService.getInstance().mapType;
            },
            enumerable: true,
            configurable: true
        });
        MapInstance.prototype.addToItemsSet = function (item) {
            var i = this.itemsSet.indexOf(item);
            if (i == -1)
                this.itemsSet.push(item);
        };
        MapInstance.prototype.renderMap = function (elem, options) {
            if (options === void 0) { options = {}; }
            var map = this.mapUtils.renderMap(elem, options);
            this.setMap(map);
            return map;
        };
        MapInstance.prototype.setMap = function (map) {
            this.map$.next(map);
        };
        MapInstance.prototype.inValidateSize = function () {
            var _this = this;
            setTimeout(function () {
                _this.mapUtils.invalidateSize(_this.map);
            }, 100);
        };
        MapInstance.prototype.getInteraction$ = function () {
            // todo move this to mapUtils for each mapType
            var mapUtils = this.mapUtils;
            var thismap = this.map;
            var mapmoveEnd$ = mapUtils.onEvent$(thismap, 'moveend');
            var mapmoveStart$ = mapUtils.onEvent$(thismap, 'movestart');
            var moveEnd$ = mapUtils.onEvent$(thismap, 'mouseup mouseout dragend');
            var move$ = mapUtils.onEvent$(thismap, 'move zoomlevelschange');
            var moveStart$ = mapUtils.onEvent$(thismap, 'click tap mousedown dragstart');
            var load$ = mapUtils.onEvent$(thismap, 'load');
            return rxjs.merge(moveStart$.pipe(operators.throttleTime(15), operators.map(function () { return true; })), moveEnd$.pipe(operators.debounceTime(15), operators.map(function () { return false; })), load$.pipe(
            // debounceTime(15),
            operators.map(function () { return false; })));
        };
        // getMap() {
        //   this.map$.take(1).subscribe(map => {
        //     return map;
        //   });
        // }
        MapInstance.prototype.setMapType = function (mapType, key) {
            mapTypeService.getInstance(mapType);
            if (key)
                this.mapUtils.setKey(key);
        };
        MapInstance.prototype.addCluster = function (cluster) {
            var _this = this;
            if (!this.clusters.includes(cluster)) {
                this.clusters.push(cluster);
                this.map$.pipe()
                    .subscribe(function (map) {
                    if (map) {
                        cluster.cluster = _this.mapUtils.getMarkerCluster(map);
                    }
                    else if (cluster.cluster) {
                        _this.mapUtils.removeClusterMarkers(cluster.cluster);
                    }
                });
            }
        };
        MapInstance.prototype.addPopper = function (popper) {
            if (!this.poppers.includes(popper)) {
                this.poppers.push(popper);
            }
            if (!this.moveEvent) {
                this.listenMove();
            }
        };
        MapInstance.prototype.removePopper = function (popper) {
            var i = this.poppers.indexOf(popper);
            if (i > -1) {
                this.poppers.splice(i, 1);
            }
        };
        MapInstance.prototype.listenMove = function () {
            var _this = this;
            this.map$.pipe(operators.filter(function (data) { return !!data; }), operators.take(1)).subscribe(function (map) {
                _this.moveEvent = _this.mapUtils.onEvent(map, 'move', function (e) {
                    _this.poppers.forEach(function (p) {
                        p.scheduleUpdate();
                    });
                });
            });
        };
        MapInstance.prototype.getBounds = function (bounds, item) {
            return item.extendBounds ? item.extendBounds(bounds) : null;
        };
        MapInstance.prototype.getItemsSetBounds = function (items) {
            var _this = this;
            var bounds = this.mapUtils.extendItemBounds();
            return items.reduce(function (bounds, item) {
                return _this.getBounds(bounds, item) || bounds;
            }, bounds);
        };
        MapInstance.prototype.resetBounds = function (options) {
            var _this = this;
            if (this.resetBoundsTimeout)
                clearTimeout(this.resetBoundsTimeout);
            this.resetBoundsTimeout = setTimeout(function () {
                var items = _this.itemsSet;
                var bounds = _this.getItemsSetBounds(items);
                if (bounds && _this.mapUtils.isValidBounds(bounds))
                    _this.setBounds(bounds, options);
            }, 40);
        };
        MapInstance.prototype.setBounds = function (bounds, options) {
            var map = this.map;
            if (!map)
                return false;
            var mapOptions = (this.mapType == "leaflet"
                ? this.leafletSetBoundsOptions
                : this.googleSetBoundsOptions);
            options =
                options || this.setBoundsOptions || mapOptions;
            this.mapUtils.setBounds(map || this.map, bounds, __assign({}, mapOptions, options));
        };
        MapInstance.prototype.onEvent$ = function (type) {
            return this.mapUtils.onEvent$(this.map, type);
        };
        MapInstance.prototype.mapEventHandler = function (type) {
            var _this = this;
            return function (handler) { return _this.mapUtils.onEvent(_this.map, type, handler); };
        };
        MapInstance.prototype.removeHandler = function (type) {
            var _this = this;
            return function (handler) { return _this.mapUtils.removeEvent(_this.map, type, handler); };
        };
        return MapInstance;
    }());

    var segmentPolylineStyles = {
        get: function (type) {
            switch (type) {
                case "google":
                    {
                        return {
                            default: {
                                strokeColor: htUtility.Color.blue,
                                strokeOpacity: 1,
                                strokeWeight: 5
                            },
                            highlight: {
                                strokeColor: htUtility.Color.blue,
                                strokeOpacity: 1,
                                strokeWeight: 5
                            },
                            fade: {
                                strokeColor: htUtility.Color.grey5,
                                strokeOpacity: 0.2,
                                strokeWeight: 2
                            }
                        };
                    }

                case "leaflet": {
                    return {
                        default: {
                            weight: 5,
                            color: htUtility.Color.blue,
                            opacity: 1
                        },
                        highlight: {
                            weight: 5,
                            color: htUtility.Color.blue,
                            opacity: 1
                        },
                        fade: {
                            weight: 4,
                            color: htUtility.Color.grey4,
                            opacity: 0.5
                        }
                    };
                }
            }
        },
    };

    var __extends = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function ExtendBoundsMixin(Base) {
        return /** @class */ (function (_super) {
            __extends(class_1, _super);
            function class_1() {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var _this = _super.apply(this, args) || this;
                _this.entities = {};
                _this.toIncludeInBounds = true;
                if (_this.toIncludeInBounds) {
                    _this.mapInstance.addToItemsSet(_this);
                }
                return _this;
            }
            class_1.prototype.extendBounds = function (bounds) {
                var _this = this;
                if (!this.toIncludeInBounds)
                    return bounds;
                var mapUtils = this.mapInstance.mapUtils;
                bounds = bounds || mapUtils.extendItemBounds();
                var newBounds = Object.keys(this.entities).reduce(function (bounds, key) {
                    var entity = _this.entities[key];
                    return _this.getBounds(entity.item, bounds);
                }, bounds);
                return newBounds;
            };
            return class_1;
        }(Base));
    }

    var __extends$1 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function MarkersMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$1(class_1, _super);
            function class_1() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.entities = {};
                return _this;
            }
            // event = new HtCustomEvent();
            class_1.prototype.getItem = function (data) {
                return this.mapInstance.mapUtils.getMarker();
            };
            class_1.prototype.getBounds = function (item, bounds) {
                return this.mapInstance.mapUtils.extendItemBounds(item, bounds, this.forceExtendBounds);
            };
            class_1.prototype.update = function (_a, positionBearing) {
                var item = _a.item, data = _a.data;
                var pathPosition = positionBearing && positionBearing.path.length ?
                    positionBearing.path[positionBearing.path.length - 1] : null;
                var position = pathPosition || this.getPosition(data);
                if (position)
                    this.mapInstance.mapUtils.updatePosition(item, position);
                if (!this.toNotSetMap)
                    this.mapInstance.mapUtils.setMap(item, this.mapInstance.map);
                var id = this.trackBy(data);
                this.event.next('update-' + id, { item: item, data: data });
            };
            return class_1;
        }(Base));
    }

    var __extends$2 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    var __assign$1 = (undefined && undefined.__assign) || Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    function StyleMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$2(class_1, _super);
            function class_1() {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var _this = _super.apply(this, args) || this;
                _this.defaultstyleFunct = {
                    get: function (type) {
                        return {
                            default: {}
                        };
                    }
                };
                _this.highlightedId = null;
                return _this;
            }
            class_1.prototype.getStyle = function (selectedStyleType, data, fallbackStyle) {
                if (selectedStyleType === void 0) { selectedStyleType = "default"; }
                var mapType = this.mapInstance.mapUtils.type;
                var styleFunct = this.styleFunct || this.defaultstyleFunct;
                var mapTypetype = this.styleObj || styleFunct.get(mapType, data);
                // console.log(this.name, "style", selectedStyleType, styleFunct, this.styleFunct);
                // const styleType = mapTypetype[selectedStyleType] ? selectedStyleType : this.styleType;
                var style = mapTypetype[selectedStyleType] || fallbackStyle;
                if (!style)
                    console.error("style type does not exist ", this.name, selectedStyleType);
                if (this.isAnimated && style)
                    style = __assign$1({}, style, { className: style['className'] ? style['className'] + " animated-marker" : 'animated-marker' });
                return style;
            };
            class_1.prototype.setStyleType = function (styleType) {
                if (styleType === void 0) { styleType = 'default'; }
                this.styleType = styleType;
            };
            class_1.prototype.setStyle = function (_a) {
                var item = _a.item, data = _a.data;
                var styleType = this.getStyleType(data);
                var style = this.getStyle(styleType, data);
                this.setItemStyle(item, style);
            };
            class_1.prototype.getStyleType = function (datum) {
                if (this.highlightedId && datum) {
                    return this.highlightedId == this.trackBy(datum) ? 'highlight' : 'fade';
                }
                else {
                    return this.styleType;
                }
            };
            class_1.prototype.setItemStyle = function (item, style) {
                this.mapInstance.mapUtils.setStyle(item, style);
            };
            return class_1;
        }(Base));
    }
    // export class Styles {
    //   styleFunct: StyleFunct = {
    //     google: {
    //       default: {
    //
    //       }
    //     },
    //     leaflet: {
    //       default: {
    //
    //       }
    //     }
    //   };
    //
    //   styleType = 'default';
    //
    //   getStyle(selectedStyleType: string = 'default', fallbackStyle?) {
    //     const mapType = GlobalMap.mapUtils.type;
    //     const mapTypetype = this.styleFunct[mapType];
    //     const styleType = selectedStyleType && mapTypetype[selectedStyleType] ? selectedStyleType : this.styleType;
    //     const style = mapTypetype[styleType] || fallbackStyle;
    //     if(!style) console.error("style type does not exist");
    //     return this.styleFunct[mapType][styleType]
    //   }
    // }

    var __extends$3 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function TraceMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$3(class_1, _super);
            function class_1() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            // setMap: (item, map) => void;
            class_1.prototype.trace = function (data, map) {
                var _this = this;
                map = map || this.mapInstance.map;
                this.event.next('update-data', data);
                if (data && data.length) {
                    if (!map) {
                        console.warn("Map is not initialized");
                        return false;
                    }
                    data.forEach(function (datum) {
                        var id = _this.trackBy(datum);
                        var entity = _this.entities[id];
                        var hasEntity = !!entity;
                        var item = !hasEntity ? _this.getItem(datum) : entity.item;
                        entity = { data: datum, item: item, isOld: false };
                        _this.entities[id] = entity;
                        if (!hasEntity) {
                            _this.addEvents(item, id);
                        }
                        if (item)
                            _this.setStyle(entity);
                        if (!_this.toNotTraceItem)
                            _this.traceItem(datum);
                    });
                }
                else {
                    this.removeAll(this.entities);
                }
                this.bustOldItem();
                if (this.traceEffect)
                    this.traceEffect();
            };
            class_1.prototype.traceItem = function (datum) {
                var id = this.trackBy(datum);
                var entity = this.entities[id];
                if (entity) {
                    this.update(entity);
                }
            };
            class_1.prototype.addEvents = function (item, id) {
                var _this = this;
                var mapUtils = this.mapInstance.mapUtils;
                mapUtils.onEvent(item, "click", function () {
                    var entity = _this.entities[id];
                    if (_this.onMouseLeave)
                        _this.onMouseLeave(entity);
                    if (_this.onClick)
                        _this.onClick(entity);
                });
                mapUtils.onEvent(item, "mouseover", function () {
                    var entity = _this.entities[id];
                    if (_this.onMouseEnter)
                        _this.onMouseEnter(entity);
                });
                mapUtils.onEvent(item, "mouseout", function () {
                    var entity = _this.entities[id];
                    if (_this.onMouseLeave)
                        _this.onMouseLeave(entity);
                });
            };
            class_1.prototype.bustOldItem = function () {
                var _this = this;
                var keys = Object.keys(this.entities);
                keys.forEach(function (key) {
                    var entity = _this.entities[key];
                    if (entity.isOld) {
                        _this.clearItem(entity);
                        delete _this.entities[key];
                    }
                    else {
                        _this.entities[key].isOld = true;
                    }
                });
            };
            return class_1;
        }(Base));
    }

    var positionTime = {
        positionTimePoints: [],
        decode: function (encodedPolyline) {
            this.positionTimePoints = new timeAwarePolyline.TimeAwareEncoder().decodeTimeAwarePolyline(encodedPolyline);
            return this.positionTimePoints;
        }
    };

    var __extends$4 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function PolylinesMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$4(class_1, _super);
            function class_1() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.positionTimeArray = [];
                return _this;
            }
            class_1.prototype.getItem = function (data) {
                return this.mapInstance.mapUtils.getPolyline();
            };
            class_1.prototype.getBounds = function (item, bounds) {
                return this.mapInstance.mapUtils.extendBoundsWithPolyline(item, bounds);
            };
            class_1.prototype.update = function (_a, pathBearing) {
                var item = _a.item, data = _a.data;
                if (this.getPath || pathBearing) {
                    var path = pathBearing ? pathBearing.path : this.getPath(data);
                    this.mapInstance.mapUtils.setPath(item, path);
                }
                else if (this.getEncodedPositionTime) {
                    this.positionTimeArray = positionTime.decode(this.getEncodedPositionTime(data));
                    this.mapInstance.mapUtils.setPathPositionTimeArray(item, this.positionTimeArray);
                }
                else {
                    this.mapInstance.mapUtils.setEncodedPath(item, this.getEncodedPath(data));
                }
                if (!this.toNotSetMap)
                    this.mapInstance.mapUtils.setMap(item, this.mapInstance.map);
            };
            class_1.prototype.setItemStyle = function (item, style) {
                this.mapInstance.mapUtils.setPolylineStyle(item, style);
            };
            return class_1;
        }(Base));
    }

    var __extends$5 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function MapItemsMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$5(class_1, _super);
            function class_1() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.entities = {};
                _this.event = new htUtility.HtCustomEvent();
                return _this;
            }
            class_1.prototype.htShow = function (item) {
                return "display: " + (item ? "flex" : "none");
            };
            class_1.prototype.getEntity = function (id) {
                if (!this.entities)
                    return null;
                if (id)
                    return this.entities[id];
                var keys = Object.keys(this.entities);
                if (keys.length == 0)
                    return null;
                var key = keys[0];
                return this.entities[key];
            };
            class_1.prototype.trackBy = function (datum) {
                return _super.prototype.trackBy ? _super.prototype.trackBy.call(this, datum) : datum.id;
            };
            class_1.prototype.trackAnimationBy = function (datum) {
                return _super.prototype.trackAnimationBy ? _super.prototype.trackAnimationBy.call(this, datum) : this.trackBy(datum);
            };
            /**
             *
             * @param id string return trackBy
             * @returns {{subscribe: (cb) => IEventSub}}
             */
            class_1.prototype.onEntityUpdate = function (id) {
                var _this = this;
                var eventName = "update-" + id;
                return {
                    subscribe: function (cb) {
                        return _this.event.subscribe(eventName, cb);
                    }
                };
            };
            class_1.prototype.removeItem = function (item) {
                this.mapInstance.mapUtils.clearItem(item);
            };
            class_1.prototype.removeAll = function (entities) {
                var _this = this;
                _.each(entities, function (entity) {
                    _this.removeItem(entity.item);
                });
                this.entities = {};
            };
            class_1.prototype.clear = function () {
                var entities = this.entities;
                this.removeAll(entities);
            };
            class_1.prototype.clearItem = function (_a) {
                var item = _a.item, data = _a.data;
                this.removeData(data);
                this.removeItem(item);
            };
            class_1.prototype.removeData = function (data) {
                var id = this.trackBy(data);
                if (this.entities[id])
                    delete this.entities[id];
            };
            return class_1;
        }(Base));
    }

    var SegmentPolylines = /** @class */ (function () {
        function SegmentPolylines(mapInstance) {
            this.mapInstance = mapInstance;
            this.styleFunct = segmentPolylineStyles;
            this.name = "polyline";
        }
        SegmentPolylines.prototype.getEncodedPath = function (data) {
            return data.route;
        };
        SegmentPolylines.prototype.getEncodedPositionTime = function (data) {
            return data.location_time_series;
        };
        SegmentPolylines.prototype.getPosition = function () {
            return { lat: 0, lng: 0 };
        };
        return SegmentPolylines;
    }());
    var SegmentPolylinesTrace = TraceMixin(ExtendBoundsMixin(PolylinesMixin(MarkersMixin(StyleMixin(MapItemsMixin(SegmentPolylines))))));
    // export class SegmentPolylines {
    //   styleFunct = segmentPolylineStyles;
    //
    //   getEncodedPath(data) {
    //     return data.encoded_polyline;
    //   };
    //
    //   getEncodedPositionTime(data: IPlaceline) {
    //     return data.time_aware_polyline
    //   }
    // }
    //
    // export const SegmentPolylinesTrace = mapItemsFactory(SegmentPolylines, {
    //   isPolyline: true,
    //   hasDataObservable: false
    // });
    // export const SegmentPolylinesTrace = _.compose(
    //   PolylinesMixin,
    //   MarkersMixin,
    //   StyleMixin,
    //   TraceMixin,
    // )(SegmentPolylines);

    var stopStyles = {
        get: function (type) {
            switch (type) {
                case 'google':
                    {
                        return {
                            default: {
                                icon: {
                                    fillColor: htUtility.Color.stop,
                                    fillOpacity: 1,
                                    strokeColor: htUtility.Color.stopDark,
                                    strokeOpacity: 1,
                                    path: google.maps.SymbolPath.CIRCLE,
                                    scale: 8,
                                    strokeWeight: 2
                                }
                            },
                            popup: {
                                disableAutoPan: true,
                                pixelOffset: new google.maps.Size(0, -10)
                            },
                            highlight: {
                                icon: {
                                    fillColor: htUtility.Color.stop,
                                    fillOpacity: 1,
                                    strokeColor: htUtility.Color.stopDark,
                                    strokeOpacity: 1,
                                    path: google.maps.SymbolPath.CIRCLE,
                                    scale: 8,
                                    strokeWeight: 2
                                }
                            },
                            fade: {
                                icon: {
                                    fillColor: htUtility.Color.stop,
                                    fillOpacity: 0.1,
                                    strokeColor: htUtility.Color.stopDark,
                                    strokeOpacity: 0.2,
                                    path: google.maps.SymbolPath.CIRCLE,
                                    scale: 8,
                                    strokeWeight: 2
                                }
                            },
                        };
                    }

                case "leaflet": {
                    return {
                        default: {
                            radius: 10,
                            fillColor: htUtility.Color.stop,
                            fillOpacity: 1,
                            weight: 2,
                            color: htUtility.Color.stopDark,
                            pane: "markerPane"
                        },
                        popup: {
                            offset: [0, -5],
                            // offset: point(0, -5),
                            closeButton: false
                        },
                        highlight: {
                            radius: 10,
                            fillColor: htUtility.Color.stop,
                            fillOpacity: 1,
                            weight: 4,
                            color: htUtility.Color.stopDark,
                            pane: "markerPane"
                        },
                        fade: {
                            radius: 10,
                            fillColor: htUtility.Color.stop,
                            fillOpacity: 0.3,
                            opacity: 0.4,
                            weight: 2,
                            color: htUtility.Color.stopDark,
                            pane: "markerPane"
                        }
                    };
                }
            }
        }
    };

    var __extends$6 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function PopupMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$6(class_1, _super);
            function class_1() {
                var arg = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    arg[_i] = arguments[_i];
                }
                var _this = _super.apply(this, arg) || this;
                _this.defaultPopupStyle = {
                    disableAutoPan: true,
                };
                _this.addPopup();
                return _this;
            }
            class_1.prototype.addPopup = function () {
                this.popup = this.mapInstance.mapUtils.getPopup(this.getStyle("popup", this.defaultPopupStyle));
            };
            class_1.prototype.setPopup = function (id) {
                if (id && this.entities[id]) {
                    var data = this.entities[id].data;
                    var popup = this.popup;
                    var map = this.mapInstance.map;
                    this.mapInstance.mapUtils.openPopupPosition(this.getPosition(data), map, this.getInfoContent(data), popup);
                }
                else {
                    this.mapInstance.mapUtils.clearItem(this.popup);
                }
            };
            class_1.prototype.onMouseEnter = function (entity) {
                var id = this.trackBy(entity.data);
                this.setPopup(id);
            };
            class_1.prototype.onMouseLeave = function (entity) {
                this.popup && this.mapInstance.mapUtils.clearItem(this.popup);
            };
            return class_1;
        }(Base));
    }
    // export class PopupRenderer {
    //   popup;
    //   getStyle: (styleType?) => object;
    //   entities: Entities<any>;
    //   getPosition: (data) => HtPosition;
    //   getInfoContent: (data) => string;
    //   // setMap: (item, map) => void;
    //   defaultPopupStyle =  {
    //     disableAutoPan: true,
    //     pixelOffset: new google.maps.Size(0, -35)
    //   };
    //
    //   addPopup() {
    //     this.popup = GlobalMap.mapUtils.getPopup(this.getStyle('popup'))
    //   }
    //
    //   setPopup(id: string | null) {
    //     if (id && this.entities[id]) {
    //       let {data} = this.entities[id];
    //       let popup = this.popup;
    //       let map = GlobalMap.map;
    //       GlobalMap.mapUtils.openPopupPosition(this.getPosition(data), map, this.getInfoContent(data), popup);
    //     } else {
    //       GlobalMap.mapUtils.setMap(this.popup, null)
    //     }
    //   };
    //
    //   onMouseEnter(entity: Entity<any>) {
    //     let id = entity.data.id;
    //     this.setPopup(id);
    //
    //   };
    //
    //   onMouseLeave(entity: Entity<any>) {
    //     this.popup && GlobalMap.mapUtils.setMap(this.popup, null)
    //   }
    // }

    var __extends$7 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function CircleMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$7(class_1, _super);
            function class_1() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            class_1.prototype.getItem = function (data) {
                return this.mapInstance.mapUtils.getCircleMarker();
            };
            return class_1;
        }(Base));
    }

    var StopMarkers = /** @class */ (function () {
        function StopMarkers(mapInstance) {
            this.mapInstance = mapInstance;
            this.styleFunct = stopStyles;
            this.name = "stop";
        }
        StopMarkers.prototype.getPosition = function (data) {
            if (data.place && data.place.location) {
                var lat = data.place.location.coordinates[1];
                var lng = data.place.location.coordinates[0];
                return { lat: lat, lng: lng };
            }
            else {
                return null;
            }
        };
        StopMarkers.prototype.getInfoContent = function (data) {
            var durationString = null;
            if (data.duration) {
                durationString = htUtility.HMString(data.duration, 60);
            }
            var start = htUtility.TimeString(data.started_at);
            var end = htUtility.TimeString(data.ended_at);
            var startDate = htUtility.DateString(data.started_at);
            var startDateShort = htUtility.DateString(data.started_at, "short");
            var endDate = htUtility.DateString(data.ended_at);
            var sameDate = startDate == endDate;
            function htShow(item) {
                return "display: " + (item ? "flex" : "none");
            }
            return "<div class=\"flex-column\">\n<strong class=\"text-muted text-center\" style=\"padding-bottom: 0; color: " + htUtility.Color.stop + "\">Stop</strong>\n<div style=\"display: " + (start ? "display" : "none") + "\" class=\"flex-row space-between\">\n    <div>" + (start ||
                "--:--") + "</div><div>&nbsp; to &nbsp;</div><div class=\"text-right\">" + (end ||
                "--:--") + "</div>\n</div>\n<div style=\"" + htShow(startDate || endDate) + "\" class=\"" + (sameDate || !endDate ? "space-around" : "space-between") + " text-2 text-muted flex-row\">\n    <div>" + startDateShort + "</div><div style=\"display: " + (sameDate || !endDate ? "none" : "block") + "\">" + endDate + "</div>\n</div>\n<div style=\"display: " + (durationString ? "block" : "none") + "\" class=\"text-3 text-center\">" + durationString + "</div>\n</div>";
        };
        return StopMarkers;
    }());
    var StopMarkersTrace = PopupMixin(TraceMixin(ExtendBoundsMixin(CircleMixin(MarkersMixin(StyleMixin(MapItemsMixin(StopMarkers)))))));
    // export const stopMarkersConfig: ItemClassFactoryConfig = {
    //   renderConfig: {
    //     getPosition(data: IPlaceline) {
    //       console.log("stop", data);
    //       if (data.place && data.place.location) {
    //         let lat = data.place.location.coordinates[1];
    //         let lng = data.place.location.coordinates[0];
    //         console.log("lat", lat, lng);
    //         return { lat, lng };
    //       } else {
    //         return null;
    //       }
    //     },
    //     getInfoContent(data: IPlaceline) {
    //       let durationString = null;
    //       if (data.duration) {
    //         durationString = HMString(data.duration, 60);
    //       }
    //
    //       let start = TimeString(data.started_at);
    //       let end = TimeString(data.ended_at);
    //       let startDate = DateString(data.started_at);
    //       let startDateShort = DateString(data.started_at, "short");
    //       let endDate = DateString(data.ended_at);
    //       let sameDate = startDate == endDate;
    //       function htShow(item) {
    //         return `display: ${item ? "flex" : "none"}`;
    //       }
    //       return `<div class="flex-column">
    // <strong class="text-muted text-center" style="padding-bottom: 0; color: ${
    //         Color.stop
    //       }">Stop</strong>
    // <div style="display: ${
    //         start ? "display" : "none"
    //       }" class="flex-row space-between">
    //     <div>${start ||
    //       "--:--"}</div><div>&nbsp; to &nbsp;</div><div class="text-right">${end ||
    //         "--:--"}</div>
    // </div>
    // <div style="${htShow(startDate || endDate)}" class="${
    //         sameDate || !endDate ? "space-around" : "space-between"
    //       } text-2 text-muted flex-row">
    //     <div>${startDateShort}</div><div style="display: ${
    //         sameDate || !endDate ? "none" : "block"
    //       }">${endDate}</div>
    // </div>
    // <div style="display: ${
    //         durationString ? "block" : "none"
    //       }" class="text-3 text-center">${durationString}</div>
    // </div>`;
    //     }
    //   },
    //   styleFunct: stopStyles,
    //   typeConfig: {
    //     hasPopup: true,
    //     isCircle: true,
    //     hasDataObservable: false
    //   },
    //   name: "stop"
    // };
    //
    // export const stopMarkersTrace = () => {
    //   return itemsFactory(stopMarkersConfig);
    // };
    // export abstract class StopMarkers {
    //   name = "Stop";
    //   styleFunct = stopStyles;
    //
    //   htShow(item) {
    //     return `display: ${item ? 'flex' : 'none'}`
    //   };
    //
    //   getPosition(data) {
    //     if(data.location && data.location.geojson) {
    //       let lat = data.location.geojson.coordinates[1];
    //       let lng = data.location.geojson.coordinates[0];
    //       return {lat, lng}
    //     } else {
    //       return null;
    //     }
    //
    //   };
    //
    //   getInfoContent(data: IPlaceline) {
    //     function htShow(item) {
    //       return `display: ${item ? 'flex' : 'none'}`
    //     };
    //     console.log(this.htShow, "htShow");
    //     let durationString = null;
    //     if(data.started_at && data.ended_at) {
    //       let durationMin = (new Date(new Date(data.ended_at).getTime() - new Date(data.started_at).getTime()).getTime()) / (1000 * 60);
    //       durationString = HMString(durationMin)
    //     }
    //
    //     let start = TimeString(data.started_at);
    //     let end = TimeString(data.ended_at);
    //     let startDate = DateString(data.started_at);
    //     let startDateShort = DateString(data.started_at, 'short');
    //     let endDate = DateString(data.ended_at);
    //     let sameDate = startDate == endDate;
    //     return `<div class="flex-column">
    // <strong class="text-muted text-center" style="padding-bottom: 0; color: ${Color.stop}">Stop</strong>
    // <div style="display: ${start ? 'display' : 'none'}" class="flex-row space-between">
    //     <div>${start || '--:--'}</div><div>&nbsp; to &nbsp;</div><div class="text-right">${end || '--:--'}</div>
    // </div>
    // <div style="${htShow(startDate || endDate)}" class="${sameDate || !endDate ? 'space-around' : 'space-between'} text-2 text-muted flex-row">
    //     <div>${startDateShort}</div><div style="display: ${sameDate || !endDate ? 'none' : 'block'}">${endDate}</div>
    // </div>
    // <div style="display: ${durationString ? 'block' : 'none'}" class="text-3 text-center">${durationString}</div>
    // </div>`
    //   }
    // }
    //
    // export const StopMarkersTrace = mapItemsFactory(StopMarkers, {
    //   hasPopup: true,
    //   isCircle: true,
    //   hasDataObservable: false
    // });

    var __extends$8 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function DivMarkersMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$8(class_1, _super);
            function class_1() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            class_1.prototype.getItem = function (data) {
                return this.mapInstance.mapUtils.getDivMarker();
            };
            class_1.prototype.update = function (_a, pathBearing) {
                var item = _a.item, data = _a.data;
                var bearing = pathBearing ? pathBearing.bearing : null;
                var content = this.getDivContent(data, bearing);
                this.setContent({ item: item, content: content });
                _super.prototype.update.call(this, { item: item, data: data }, pathBearing);
            };
            class_1.prototype.setContent = function (_a) {
                var item = _a.item, content = _a.content;
                this.mapInstance.mapUtils.setDivContent(item, content, this.getStyle());
            };
            class_1.prototype.setStyle = function (entity) {
                // let style = this.getStyle();
                // GlobalMap.mapUtils.setDivMarkerStyle(item, style)
            };
            return class_1;
        }(Base));
    }

    var ActionMarkers = /** @class */ (function () {
        function ActionMarkers(mapInstance) {
            this.mapInstance = mapInstance;
            this.name = "Action";
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case 'google': {
                            return {
                                default: {
                                    flat: true,
                                    anchor: RichMarkerPosition.MIDDLE,
                                    zIndex: 1
                                },
                                popup: {
                                    pixelOffset: new google.maps.Size(0, -5),
                                }
                            };
                        }
                        case 'leaflet': {
                            return {
                                default: {
                                    iconAnchor: [12, 12]
                                    // iconSize: [35, 35],
                                    // className: 'current-action-marker',
                                    // iconAnchor: point(15, 43)
                                    // iconAnchor: [15, 43]
                                },
                                popup: {
                                    // offset: point(0, -35),
                                    offset: [0, -5],
                                    closeButton: false
                                }
                            };
                        }
                    }
                }
            };
        }
        ActionMarkers.prototype.getPosition = function (data) {
            var posObj = htData.htAction(data).getPositionsObject();
            return posObj ? posObj.position : null;
        };
        ActionMarkers.prototype.getDivContent = function (action) {
            var actionClass = "";
            if (action.completed_at)
                actionClass = 'action-marker-filled';
            if (action.display.is_late)
                actionClass = actionClass + " is-late";
            var icon = "<div class=\"action-marker " + actionClass + " flex-row\">\n<span style=\"margin: auto\">" + htUtility.NameCase(action.type[0]) + "</span>\n</div>";
            return icon;
        };
        return ActionMarkers;
    }());
    var ActionMarkersTrace = ExtendBoundsMixin(TraceMixin(DivMarkersMixin(MarkersMixin(StyleMixin(MapItemsMixin(ActionMarkers))))));

    var userDivFactory = function (user) {
        return "<div class=\"m-marker\">\n        <div class=\"m-box\" style=\"background-image: url('" + user.photo + "')\"></div>\n          <i class=\"fa fa-neuter\"></i>\n      </div>";
    };

    var __extends$9 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function SingleItemMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$9(class_1, _super);
            function class_1() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            class_1.prototype.trace = function (user, map) {
                var data = user ? [user] : [];
                _super.prototype.trace.call(this, data, map);
            };
            return class_1;
        }(Base));
    }

    var __extends$10 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function AnimationMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$10(class_1, _super);
            function class_1() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                // bearing: number = 0;
                // item;
                _this.isAnimated = true;
                return _this;
            }
            class_1.prototype.setTimeAwareAnimation = function (animation) {
                var _this = this;
                this.animation = animation || new timeAwarePolyline.TimeAwareAnimation();
                this.animation.updateEvent.subscribe('update', function (_a) {
                    var path = _a.path, bearing = _a.bearing;
                    var entity = _this.getEntity();
                    if (entity)
                        _this.update(entity, { path: path, bearing: bearing });
                });
            };
            return class_1;
        }(Base));
    }

    var CurrentUser = /** @class */ (function () {
        function CurrentUser(mapInstance) {
            this.mapInstance = mapInstance;
            this.name = "Current user";
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case "google":
                            {
                                return {
                                    default: {
                                        flat: true,
                                        anchor: RichMarkerPosition.BOTTOM_CENTER
                                    }
                                };
                            }

                        case "leaflet": {
                            return {
                                default: {
                                    // iconAnchor: point(15, 43)
                                    iconAnchor: [15, 43]
                                }
                            };
                        }
                    }
                }
            };
        }
        CurrentUser.prototype.getPosition = function (data) {
            return htData.htUser(data).getPosition();
        };
        CurrentUser.prototype.getDivContent = function (data) {
            return userDivFactory(data);
        };
        CurrentUser.prototype.isValidMapItems = function (data) {
            return htData.htUser(data).isValidMarker();
        };
        return CurrentUser;
    }());
    var CurrentUserTrace = AnimationMixin(SingleItemMixin(DivMarkersMixin(TraceMixin(MarkersMixin(StyleMixin(MapItemsMixin(CurrentUser)))))));
    // export const currentUserConfig: ItemClassFactoryConfig = {
    //   renderConfig: {
    //     getPosition(data): HtPosition {
    //       return htUser(data).getPosition();
    //     },
    //     getDivContent(data) {
    //       return userDivFactory(data);
    //     }
    //   },
    //
    //   styleFunct: {
    //     get(type) {
    //       switch (type) {
    //         case "google": {
    //           return {
    //             default: {
    //               flat: true,
    //               anchor: RichMarkerPosition.BOTTOM_CENTER
    //             }
    //           }
    //         };
    //         case "leaflet": {
    //           return {
    //             default: {
    //               // iconAnchor: point(15, 43)
    //               iconAnchor: [15, 43]
    //             }
    //           }
    //         }
    //       }
    //     }
    //   },
    //   typeConfig: {
    //     isSingleItem: true,
    //     isDiv: true,
    //     hasDataObservable: false
    //   },
    //   name: "Current user"
    // };
    //
    // export const currentUserTrace = () => {
    //   return itemsFactory(currentUserConfig);
    // };
    // export class CurrentUser {
    //   name = "Current user";
    //   styleFunct: StyleFunct = {
    //     google: {
    //       default: {
    //         flat: true,
    //         anchor: RichMarkerPosition.BOTTOM_CENTER,
    //       }
    //     },
    //     leaflet: {
    //       default: {
    //
    //       }
    //     }
    //   };
    //
    //   getPosition(data): HtPosition {
    //     return htUser(data).getPosition()
    //   };
    //
    //   getDivContent(data) {
    //     return userDivFactory(data)
    //
    //   }
    // };
    //
    // export const CurrentUserTrace = mapItemsFactory(CurrentUser, {
    //   isSingleItem: true,
    //   isDiv: true,
    //   hasDataObservable: false
    // });

    var __extends$11 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function CompoundDataObservableMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$11(class_1, _super);
            function class_1() {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var _this = _super.apply(this, args) || this;
                _this.mapInstance.addToItemsSet(_this);
                return _this;
            }
            class_1.prototype._procData$ = function () {
                return function (source$) {
                    return source$.pipe(operators.map(function (markers) {
                        return markers.reduce(function (acc, item) {
                            var isValid = true;
                            if (isValid) {
                                acc.valid.push(item);
                            }
                            else {
                                acc.invalid.push(item);
                            }
                            return acc;
                        }, { valid: [], invalid: [] });
                        // return markers
                    }));
                };
            };
            class_1.prototype.setCompoundData$ = function (data$, config) {
                if (config === void 0) { config = {}; }
                this.compoundSetDataConfig = config;
                if (this.dataSub) {
                    this.dataSub.unsubscribe();
                }
                var hide$ = config.hide$;
                var filter$ = config.filter$;
                var dataSource$ = hide$
                    ? rxjs.combineLatest(data$, hide$.pipe(operators.distinctUntilChanged()), function (data, hide) { return (!!hide ? null : data); })
                    : data$;
                if (config.roots && config.highlighted$) {
                    dataSource$ = htData.dataWithSelectedId$(dataSource$, config.highlighted$, config.roots, 'highlightedSegment');
                }
                if (config.roots && filter$)
                    dataSource$ = htData.dataWithSelectedId$(dataSource$, filter$, config.roots, 'selectedSegment');
                this.dataSource$ = dataSource$;
                this.data$ = this.dataSource$;
                this._initDataObserver();
            };
            class_1.prototype._initData$ = function () {
                var mapData$ = this.data$;
                var render$ = rxjs.combineLatest(mapData$, this.mapInstance.map$).pipe(operators.map(function (_a) {
                    var mapData = _a[0], map = _a[1];
                    return map ? mapData : null;
                }));
                // function isNewId (newItem, old) {
                //   if(!old && newItem) return true;
                //   if(newItem && old) return  newItem.id !== old.id
                // }
                // function isNewList(newList, old) {
                //   if(!old && newList) return true;
                //   if(newList && old) return !newList.next && newList.count
                // }
                // let sub = render$.subscribe(({ valid, invalid, isNew }) => {
                //   this.trace(valid);
                //   if (isNew) this.mapInstance.resetBounds();
                // });
                // this.dataSub = sub;
                // here
                var userData$ = render$.pipe(operators.scan(function (acc, data) {
                    var oldUser = acc.user;
                    return { user: data, oldUser: oldUser };
                }, { user: null, oldUser: null }));
                return userData$;
            };
            class_1.prototype._initDataObserver = function () {
                var _this = this;
                var userData$ = this._initData$();
                var toResetMap = this.toResetMap;
                function isNewItem(newItem, old) {
                    if (toResetMap)
                        return toResetMap(newItem, old);
                    if (newItem && old) {
                        return newItem.id !== old.id || (newItem.timeline_date !== old.timeline_date);
                    }
                    else if (!old && newItem) {
                        return true;
                    }
                    else {
                        return false;
                    }
                }
                var newPlaceline$ = userData$.pipe(operators.map(function (acc) {
                    var userData = acc.user;
                    _this.trace(userData);
                    var isNew = isNewItem(acc.user, acc.oldUser);
                    return isNew;
                    // if(isNew) GlobalMap.resetBounds()
                }));
                var resetMap$ = this.compoundSetDataConfig.resetMap$ ?
                    this.compoundSetDataConfig.resetMap$.pipe(operators.map(function (data) { return true; })) : rxjs.of(true);
                var sub = htData.orCombine(newPlaceline$.pipe(operators.filter(function (data) { return !!data; })), resetMap$).subscribe(function (toReset) {
                    if (toReset)
                        _this.mapInstance.resetBounds();
                });
                this.dataSub = sub;
            };
            return class_1;
        }(Base));
    }

    var __extends$12 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    var AnimPolyline = /** @class */ (function (_super) {
        __extends$12(AnimPolyline, _super);
        function AnimPolyline() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.name = "anim polyline";
            return _this;
        }
        return AnimPolyline;
    }(SegmentPolylines));
    var AnimPolylineTrace = AnimationMixin(SingleItemMixin(TraceMixin(ExtendBoundsMixin(PolylinesMixin(MarkersMixin(StyleMixin(MapItemsMixin(AnimPolyline))))))));

    var __extends$13 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    var ActionsPolyline = /** @class */ (function (_super) {
        __extends$13(ActionsPolyline, _super);
        function ActionsPolyline() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.lineSymbol = {
                path: 'M 0,-1 0,1',
                strokeOpacity: 1,
                scale: 4
            };
            _this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case "google":
                            {
                                return {
                                    default: {
                                        strokeColor: htUtility.Color.grey3,
                                        strokeOpacity: 0,
                                        strokeWeight: 5,
                                        icons: [{
                                                icon: {
                                                    path: 'M 0,-1 0,0',
                                                    strokeOpacity: 1,
                                                    scale: 4
                                                },
                                                offset: '0',
                                                repeat: '13px'
                                            }],
                                    },
                                    highlight: {
                                        strokeColor: htUtility.Color.grey2,
                                        strokeOpacity: 1,
                                        strokeWeight: 5
                                    },
                                    fade: {
                                        strokeColor: htUtility.Color.grey2,
                                        strokeOpacity: 0.2,
                                        strokeWeight: 2
                                    }
                                };
                            }

                        case "leaflet": {
                            return {
                                default: {
                                    weight: 5,
                                    color: htUtility.Color.grey4,
                                    opacity: 1,
                                    dashArray: "7 10"
                                },
                                highlight: {
                                    weight: 5,
                                    color: htUtility.Color.grey2,
                                    opacity: 1
                                },
                                fade: {
                                    weight: 2,
                                    color: htUtility.Color.grey2,
                                    opacity: 0.2
                                }
                            };
                        }
                    }
                },
            };
            _this.name = "action polyline";
            return _this;
        }
        ActionsPolyline.prototype.getPathFromData = function (data) {
            return data ? data.actions.reduce(function (path, action) {
                return htData.htAction(action).getExpectedPosition() && !action.completed_at ? path.concat([htData.htAction(action).getPosition()]) : path;
            }, []) : null;
        };
        return ActionsPolyline;
    }(SegmentPolylines));
    var ActionsPolylineTrace = ConnectorMixin(AnimationMixin(SingleItemMixin(TraceMixin(ExtendBoundsMixin(PolylinesMixin(MarkersMixin(StyleMixin(MapItemsMixin(ActionsPolyline)))))))));
    function ConnectorMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$13(class_1, _super);
            function class_1() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.positionTimeArray = [];
                return _this;
            }
            /*
            Need to ensure connector is traced before this class is traced
             */
            class_1.prototype.setConnector = function (entity) {
                this.connector = entity ? entity.item : null;
            };
            class_1.prototype.getConnectorPosition = function () {
                return this.connector ? this.mapInstance.mapUtils.getItemPosition(this.connector) : null;
            };
            class_1.prototype.update = function (_a, pathBearing) {
                var item = _a.item, data = _a.data;
                var startPosition;
                if (pathBearing) {
                    var path = pathBearing.path;
                    startPosition = path[path.length - 1];
                }
                else {
                    startPosition = this.getConnectorPosition();
                }
                var dataPath = this.getPathFromData(data);
                if (dataPath && startPosition) {
                    var finalPath = [startPosition].concat(dataPath);
                    this.mapInstance.mapUtils.setPath(item, finalPath);
                    this.mapInstance.mapUtils.setMap(item, this.mapInstance.map);
                }
                else {
                    this.clear();
                }
            };
            return class_1;
        }(Base));
    }

    var StopsPolyline = /** @class */ (function () {
        function StopsPolyline(mapInstance) {
            this.mapInstance = mapInstance;
            this.name = "stop polyline";
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case "google":
                            {
                                return {
                                    default: {
                                        strokeColor: htUtility.Color.blue,
                                        strokeOpacity: 1,
                                        strokeWeight: 5
                                    },
                                    highlight: {
                                        strokeColor: htUtility.Color.blue,
                                        strokeOpacity: 1,
                                        strokeWeight: 5
                                    },
                                    fade: {
                                        strokeColor: htUtility.Color.grey5,
                                        strokeOpacity: 0.2,
                                        strokeWeight: 2
                                    }
                                };
                            }

                        case "leaflet": {
                            return {
                                default: {
                                    weight: 3,
                                    color: htUtility.Color.stopDark,
                                    opacity: 1
                                },
                                highlight: {
                                    weight: 4,
                                    color: htUtility.Color.stopDark,
                                    opacity: 1
                                },
                                fade: {
                                    weight: 4,
                                    color: htUtility.Color.grey4,
                                    opacity: 0.5
                                }
                            };
                        }
                    }
                },
            };
        }
        StopsPolyline.prototype.getEncodedPath = function (data) {
            return data.route;
        };
        return StopsPolyline;
    }());
    var StopsPolylineTrace = TraceMixin(PolylinesMixin(StyleMixin(MapItemsMixin(StopsPolyline))));

    var Placeline = /** @class */ (function () {
        function Placeline(options) {
            this.options = options;
            this.allowedEvents = {};
            this.anim = new timeAwarePolyline.TimeAwareAnimation();
            this.mapInstance = this.options.mapInstance;
            this.stopMarkers = new StopMarkersTrace(this.mapInstance);
            this.userMarker = new CurrentUserTrace(this.mapInstance);
            this.userMarker.setTimeAwareAnimation(this.anim);
            this.segmentsPolylines = new SegmentPolylinesTrace(this.mapInstance);
            this.actionMarkers = new ActionMarkersTrace(this.mapInstance);
            this.animPolyline = new AnimPolylineTrace(this.mapInstance);
            this.animPolyline.setTimeAwareAnimation(this.anim);
            this.actionsPolyline = new ActionsPolylineTrace(this.mapInstance);
            this.actionsPolyline.setTimeAwareAnimation(this.anim);
            this.stopsPolyline = new StopsPolylineTrace(this.mapInstance);
        }
        Object.defineProperty(Placeline.prototype, "map", {
            get: function () {
                return this.mapInstance.map;
            },
            enumerable: true,
            configurable: true
        });
        Placeline.prototype.trace = function (user, map) {
            if (user) {
                var selectedSegment = user ? user.selectedSegment : null;
                this.setHighlightId(user);
                var userSegments = user && user.placeline ? user.placeline : [];
                var segType = this.getSegmentTypes(userSegments);
                var lastSegment = segType.lastSegment;
                var restTrips = void 0;
                if (segType.lastIsTrip)
                    restTrips = segType.tripSegment.pop();
                this.traceStops(segType.stopSegment, selectedSegment, lastSegment);
                if (lastSegment) {
                    var string = this.getTimeAwarePolyline(lastSegment);
                    if (string) {
                        //todo infer toNotTraceItem from animMixin trace
                        this.userMarker.toNotTraceItem = true;
                        this.animPolyline.toNotTraceItem = true;
                        this.actionsPolyline.toNotTraceItem = true;
                        // this.animPolyline.trace(restTrips);
                        this.anim.updatePolylineString(string);
                    }
                    else {
                        this.animPolyline.toNotTraceItem = false;
                        this.userMarker.toNotTraceItem = false;
                        this.actionsPolyline.toNotTraceItem = false;
                        // if (!selectedSegment) this.animPolyline.trace(restTrips);
                        this.anim.clear();
                    }
                    if (this.userMarker.isValidMapItems(user)) {
                        this.userMarker.trace(user);
                    }
                    else {
                        this.userMarker.clear();
                    }
                    if (restTrips)
                        this.traceAnimPolyline(restTrips, selectedSegment);
                    this.actionsPolyline.setConnector(this.userMarker.getEntity());
                    this.actionsPolyline.trace(user);
                }
                else {
                    this.anim.clear();
                    this.userMarker.clear();
                    this.animPolyline.clear();
                    this.actionsPolyline.clear();
                }
                this.traceSegments(segType.tripSegment, selectedSegment);
                this.traceAction(user, selectedSegment);
                this.traceStopsPolyline(user, selectedSegment);
            }
            else {
                this.userMarker.clear();
                this.animPolyline.clear();
                this.actionsPolyline.clear();
                this.userMarker.clear();
                this.stopMarkers.clear();
                this.actionMarkers.clear();
                this.segmentsPolylines.clear();
                this.stopsPolyline.clear();
                this.anim.clear();
            }
            // this.actionsPolyline.setConnector(this.userMarker.getEntity());
            // this.actionsPolyline.trace(user)
        };
        Placeline.prototype.traceStopsPolyline = function (user, selectedSegment) {
            var stopsWithPolyline = user ? user.placeline.filter(function (placeline) {
                return placeline.type == 'stop' && !!placeline.route && (!selectedSegment || selectedSegment.id == placeline.id);
            }) : [];
            this.stopsPolyline.trace(stopsWithPolyline);
        };
        Placeline.prototype.traceAnimPolyline = function (restTrip, selectedSegment) {
            if (!restTrip) {
                this.animPolyline.clear();
            }
            else if (!selectedSegment || selectedSegment.id === restTrip.id) {
                this.animPolyline.trace(restTrip);
            }
            else {
                this.animPolyline.clear();
            }
        };
        Placeline.prototype.traceStops = function (stops, selectedSegment, lastSegment) {
            if (selectedSegment) {
                stops = selectedSegment.type == 'stop' ? [selectedSegment] : [];
            }
            this.stopMarkers.trace(stops);
        };
        Placeline.prototype.traceSegments = function (trips, selectedSegment) {
            if (trips === void 0) { trips = []; }
            if (selectedSegment) {
                var matchedTrip = trips.find(function (trip) { return trip.id === selectedSegment.id; });
                trips = matchedTrip ? [matchedTrip] : [];
            }
            this.segmentsPolylines.trace(trips);
        };
        Placeline.prototype.traceAction = function (user, selectedSegment) {
            var actions = user && user.actions && !selectedSegment ? user.actions : [];
            var filteredActions = actions.filter(function (action) {
                return htData.htAction(action).isValidMarker();
            });
            if (this.actionMarkers)
                this.actionMarkers.trace(filteredActions);
        };
        Placeline.prototype.setHighlightId = function (user) {
            var data = user && !user.selectedSegment ? user.highlightedSegment : null;
            var id = data ? data.id : null;
            this.stopMarkers.highlightedId = id;
            this.stopsPolyline.highlightedId = id;
            this.segmentsPolylines.highlightedId = id;
            this.animPolyline.highlightedId = id;
        };
        Placeline.prototype.getTimeAwarePolyline = function (segment) {
            return segment ? segment.location_time_series : null;
        };
        Placeline.prototype.extendBounds = function (bounds) {
            bounds = this.stopMarkers.extendBounds(bounds);
            bounds = this.segmentsPolylines.extendBounds(bounds);
            bounds = this.animPolyline.extendBounds(bounds);
            bounds = this.actionMarkers.extendBounds(bounds);
            // bounds = this.userMarker.extendBounds(bounds);
            // console.log(bounds, "final");
            return bounds;
        };
        Placeline.prototype.getSegmentTypes = function (userSegments) {
            return userSegments.reduce(function (segmentType, segment) {
                segmentType.lastSegment = segment;
                if (segment.type == "stop") {
                    segmentType.lastIsTrip = false;
                    if (segment.place && segment.place.location.coordinates)
                        segmentType.stopSegment.push(segment);
                }
                else {
                    if (segment.route) {
                        segmentType.tripSegment.push(segment);
                        segmentType.lastIsTrip = true;
                    }
                }
                return segmentType;
            }, { tripSegment: [], stopSegment: [], lastSegment: null, lastIsTrip: false });
        };
        return Placeline;
    }());
    var PlacelineTrace = CompoundDataObservableMixin(Placeline);

    var __extends$14 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function DataObservableMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$14(class_1, _super);
            function class_1() {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return _super.apply(this, args) || this;
            }
            class_1.prototype._trace = function (data, map) {
                if (data === void 0) { data = []; }
                if (!this.traceData$) {
                    this.traceData$ = new rxjs.BehaviorSubject(data);
                }
                else {
                    this.traceData$.next(data);
                }
                this.setData$(this.traceData$);
            };
            class_1.prototype._procData$ = function () {
                var _this = this;
                return function (source$) {
                    return source$.pipe(
                    // filter(data => !!data),
                    operators.map(function (pageData) {
                        if (pageData) {
                            var isNew = pageData && pageData.count && !pageData.previous;
                            var results = pageData ? pageData.results : [];
                            return _this.getMarkersArray(results, isNew);
                        }
                        else {
                            return _this.emptyRenderData();
                        }
                    }));
                };
            };
            class_1.prototype.getMarkersArray = function (markersArray, isNew) {
                var _this = this;
                if (markersArray === void 0) { markersArray = []; }
                if (isNew === void 0) { isNew = false; }
                return markersArray.reduce(function (acc, item) {
                    var isValid = _this.isValidMapItems
                        ? _this.isValidMapItems(item)
                        : !!_this.getPosition(item);
                    if (isValid) {
                        acc.valid.push(item);
                    }
                    else {
                        acc.invalid.push(item);
                    }
                    return acc;
                }, { valid: [], invalid: [], isNew: isNew });
            };
            class_1.prototype.setPageData$ = function (data$, config) {
                if (config === void 0) { config = {}; }
                if (this.dataSub) {
                    this.dataSub.unsubscribe();
                }
                var hide$ = config.hide$;
                this.dataPageSource$ = hide$
                    ? rxjs.combineLatest(data$, hide$.pipe(operators.distinctUntilChanged()), function (data, hide) {
                        return !!hide ? { results: [], count: 0, next: "", previous: "" } : data;
                    })
                    : data$;
                this.data$ = this.dataPageSource$.pipe(this._procData$());
                this._initDataObserver();
            };
            class_1.prototype.setData$ = function (data$, config) {
                var _this = this;
                if (config === void 0) { config = {}; }
                if (this.dataSub) {
                    this.dataSub.unsubscribe();
                }
                var hide$ = config.hide$;
                this.dataArraySource$ = hide$
                    ? rxjs.combineLatest(data$, hide$.pipe(operators.distinctUntilChanged()), function (data, hide) { return (!!hide ? [] : data); })
                    : data$;
                this.data$ = this.dataArraySource$.pipe(operators.scan(function (acc, data) {
                    return { new: data, old: acc.new };
                }, { old: null, new: null }), operators.map(function (dataArray) {
                    return _this.getMarkersArray(dataArray.new || [], !dataArray.old);
                }));
                this._initDataObserver();
            };
            // _initData$() {
            //   let userData$ = this.dataPageSource$.pipe(
            //     filter(data => !!GlobalMap.map),
            //     pluck('valid'),
            //     scan((acc: {user: any, oldUser: any}, data: object) => {
            //       const oldUser = acc.user;
            //       return {user: data, oldUser }
            //     }, {user: null, oldUser: null})
            //   );
            //   return userData$;
            // };
            class_1.prototype._initDataObserver = function () {
                var _this = this;
                var mapData$ = this.data$;
                var render$ = rxjs.combineLatest(mapData$, this.mapInstance.map$).pipe(operators.map(function (_a) {
                    var mapData = _a[0], map = _a[1];
                    return map ? mapData : _this.emptyRenderData();
                }));
                // function isNewId (newItem, old) {
                //   if(!old && newItem) return true;
                //   if(newItem && old) return  newItem.id !== old.id
                // }
                // function isNewList(newList, old) {
                //   if(!old && newList) return true;
                //   if(newList && old) return !newList.next && newList.count
                // }
                var sub = render$.subscribe(function (_a) {
                    var valid = _a.valid, invalid = _a.invalid, isNew = _a.isNew;
                    _this.trace(valid);
                    if (isNew)
                        _this.mapInstance.resetBounds();
                });
                this.dataSub = sub;
            };
            class_1.prototype.emptyRenderData = function () {
                return { valid: [], invalid: [], isNew: false };
            };
            class_1.prototype.clear = function () {
                this.dataSub.unsubscribe();
            };
            return class_1;
        }(Base));
    }

    var __extends$15 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function ClusterMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$15(class_1, _super);
            function class_1() {
                var arg = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    arg[_i] = arguments[_i];
                }
                var _this = _super.apply(this, arg) || this;
                _this.forceExtendBounds = true;
                _this.toNotSetMap = true;
                _this.addCluster();
                _this.event.subscribe('update-data', function (data) {
                    _this.clearOnNew(data);
                });
                return _this;
            }
            class_1.prototype.clearOnNew = function (items) {
                if (items && items.length) {
                    this.clearAllClusters(items);
                }
            };
            class_1.prototype.addCluster = function () {
                this.mapInstance.addCluster(this);
            };
            class_1.prototype.traceEffect = function () {
                if (this.cluster) {
                    var userMarkerArray = _.values(this.entities).map(function (userMarker) {
                        return userMarker.item;
                    });
                    this.mapInstance.mapUtils.addMarkersToCluster(this.cluster, userMarkerArray, this.mapInstance.map);
                }
            };
            class_1.prototype.getBounds = function (item, bounds) {
                return this.mapInstance.mapUtils.extendItemBounds(item, bounds, true);
            };
            class_1.prototype.removeItem = function (item) {
                this.mapInstance.mapUtils.removeClusterMarker(this.cluster, item);
                _super.prototype.removeItem.call(this, item);
            };
            class_1.prototype.removeAll = function (entities) {
                this.cluster && this.mapInstance.mapUtils.removeClusterMarkers(this.cluster);
                // this.entities = {}
                _super.prototype.removeAll.call(this, entities);
            };
            class_1.prototype.clearAllClusters = function (data) {
                var entitiesCount = Object.keys(this.entities).length;
                if (entitiesCount > 400 && entitiesCount - data.length > 100) {
                    this.removeAll(this.entities);
                }
            };
            return class_1;
        }(Base));
    }

    var UsersCluster = /** @class */ (function () {
        function UsersCluster(mapInstance) {
            this.mapInstance = mapInstance;
            this.name = "users cluster";
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case 'google': {
                            return {
                                default: {
                                    flat: true,
                                    anchor: RichMarkerPosition.BOTTOM_CENTER,
                                    zIndex: 1
                                },
                                popup: {
                                    disableAutoPan: true,
                                    pixelOffset: new google.maps.Size(0, -35)
                                }
                            };
                        }
                        case 'leaflet': {
                            return {
                                default: {
                                    // iconAnchor: point(15, 43)
                                    iconAnchor: [15, 43]
                                },
                                popup: {
                                    // offset: point(0, -35),
                                    offset: [0, -35],
                                    closeButton: false
                                }
                            };
                        }
                    }
                }
            };
        }
        UsersCluster.prototype.getPosition = function (data) {
            return htData.htUser(data).getPosition();
        };
        UsersCluster.prototype.getDivContent = function (data) {
            return userDivFactory(data);
        };
        UsersCluster.prototype.getInfoContent = function (data) {
            var string = "<div>\n<strong>" + data.name + "</strong>\n<div>" + data.display.status_text + "</div>\n<div>" + data.display.last_updated_text + "</div>\n</div>";
            return string;
        };
        UsersCluster.prototype.traceEffect = function () {
        };
        return UsersCluster;
    }());
    var UsersClusterTrace = DataObservableMixin(PopupMixin(ClusterMixin(TraceMixin(ExtendBoundsMixin(DivMarkersMixin(MarkersMixin(StyleMixin(MapItemsMixin(UsersCluster)))))))));
    // export class UsersClusters {
    //   name = "Cluster user";
    //   styleFunct: StyleFunct = {
    //     google: {
    //       default: {
    //         flat: true,
    //         anchor: RichMarkerPosition.BOTTOM_CENTER,
    //         zIndex: 1
    //       },
    //       popup: {
    //         disableAutoPan: true,
    //         pixelOffset: new google.maps.Size(0, -35)
    //       }
    //     },
    //     leaflet: {
    //       default: {
    //
    //       }
    //     }
    //   };
    //
    //   getPosition(data): HtPosition {
    //     return htUser(data).getPosition()
    //   };
    //
    //   getDivContent(data) {
    //     return userDivFactory(data)
    //   };
    //
    //   getInfoContent(data) {
    //     let string = `<div>
    // <strong>${data.name}</strong>
    // <div>${data.display.status_text}</div>
    // <div>${data.display.sub_status_text}</div>
    // </div>`;
    //     return string
    //   }
    // }
    //
    // export const UsersClustersTrace = mapItemsFactory(UsersClusters, {
    //   hasPopup: true,
    //   isCluster: true,
    //   isDiv: true,
    //   hasDataObservable: true
    // });

    var __extends$16 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function HeatmapMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$16(class_1, _super);
            function class_1() {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var _this = _super.apply(this, args) || this;
                _this.forceExtendBounds = true;
                var style = _this.getStyle();
                _this.heatmap = _this.mapInstance.mapUtils.getHeatmap(style);
                return _this;
            }
            class_1.prototype.trace = function (items, map) {
                var _this = this;
                this.entities = items.map(function (item) { return ({ item: item }); });
                this.map = map || this.mapInstance.map;
                if (items && items.length) {
                    if (!this.map) {
                        console.warn("Map is not initialized");
                        return false;
                    }
                    var latLngs = items.reduce(function (acc, item) {
                        var position = _this.getPosition(item);
                        return position ? acc.concat([position]) : acc;
                    }, []);
                    this.mapInstance.mapUtils.updateHeatMapLatlng(latLngs, this.heatmap);
                    this.mapInstance.mapUtils.setMap(this.heatmap, this.map);
                }
                else {
                    this.clear();
                }
            };
            class_1.prototype.getBounds = function (item, bounds) {
                var position = this.getPosition(item);
                return this.mapInstance.mapUtils.extendBounds(position, bounds);
            };
            class_1.prototype.clear = function () {
                this.trace([]);
            };
            return class_1;
        }(Base));
    }

    var StopsHeatmap = /** @class */ (function () {
        function StopsHeatmap(mapInstance) {
            this.name = "stop heatmap";
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case "google": {
                            return {
                                default: {}
                            };
                        }
                        case "leaflet": {
                            return {
                                default: {
                                    radius: 7,
                                    minOpacity: 0.6
                                }
                            };
                        }
                    }
                }
            };
            this.mapInstance = mapInstance;
        }
        StopsHeatmap.prototype.getPosition = function (item) {
            if (item.place__location) {
                return {
                    lat: item.place__location[1],
                    lng: item.place__location[0],
                    weight: item.intensity
                };
            }
            else {
                return null;
            }
        };
        return StopsHeatmap;
    }());
    var StopsHeatmapTrace = ExtendBoundsMixin(DataObservableMixin(HeatmapMixin(StyleMixin(MapItemsMixin(StopsHeatmap)))));

    var ActionsCluster = /** @class */ (function () {
        function ActionsCluster(mapInstance) {
            this.mapInstance = mapInstance;
            this.name = "action cluster";
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case 'google': {
                            return {
                                default: {
                                    flat: true,
                                    anchor: RichMarkerPosition.MIDDLE,
                                    zIndex: 1
                                },
                                popup: {
                                    pixelOffset: new google.maps.Size(0, -5),
                                }
                            };
                        }
                        case 'leaflet': {
                            return {
                                default: {
                                    iconAnchor: [12, 12]
                                    // iconSize: [35, 35],
                                    // className: 'current-action-marker',
                                    // iconAnchor: point(15, 43)
                                    // iconAnchor: [15, 43]
                                },
                                popup: {
                                    // offset: point(0, -35),
                                    offset: [0, -5],
                                    closeButton: false
                                }
                            };
                        }
                    }
                }
            };
        }
        ActionsCluster.prototype.getPosition = function (data) {
            return htData.htAction(data).getPosition();
        };
        ActionsCluster.prototype.getDivContent = function (action) {
            var actionClass = "";
            if (action.completed_at)
                actionClass = 'action-marker-filled';
            if (action.display.is_late)
                actionClass = actionClass + " is-late";
            var icon = "<div class=\"action-marker " + actionClass + " flex-row\">\n<span style=\"margin: auto\">" + htUtility.NameCase(action.type[0]) + "</span>\n</div>";
            return icon;
        };
        ActionsCluster.prototype.getInfoContent = function (item) {
            function htShow(item, style) {
                if (style === void 0) { style = "flex"; }
                return "display: " + (item ? style : "none");
            }
            var userName = item.user ? item.user.name : '';
            return "<div class=\"flex-column flex-center\" style=\"min-width: 180px\">\n<div class=\"\">\n    <div class=\"text-center\">" + htUtility.NameCase(item.type) + "\n    <span style=\"" + htShow(item.display.duration_remaining && !item.display.show_summary) + "\"> in " + htUtility.HMString(item.display.duration_remaining / 60) + "</span>\n    <span style=\"" + htShow(!!item.completed_at) + "\"> completed at " + htUtility.TimeString(item.completed_at) + "</span>\n   \n    </div>\n</div>\n    <div class=\"text-muted text-center\" style=\"" + htShow(!!item.completed_at) + "\"> " + htUtility.DateString(item.completed_at) + "</div>\n<div class=\"text-center\">" + htUtility.NameCase(userName) + "<span style=\"" + htShow(!!item.lookup_id, 'block') + "\"> | #" + item.lookup_id + "</span></div>\n</div>";
        };
        ActionsCluster.prototype.traceEffect = function () {
        };
        return ActionsCluster;
    }());
    var ActionsClusterTrace = DataObservableMixin(PopupMixin(ClusterMixin(TraceMixin(ExtendBoundsMixin(DivMarkersMixin(MarkersMixin(StyleMixin(MapItemsMixin(ActionsCluster)))))))));

    var ActionsHeatmap = /** @class */ (function () {
        function ActionsHeatmap(mapInstance) {
            this.mapInstance = mapInstance;
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case "google": {
                            return {
                                default: {}
                            };
                        }
                        case "leaflet": {
                            return {
                                default: {
                                    radius: 7,
                                    minOpacity: 0.6
                                }
                            };
                        }
                    }
                }
            };
            this.name = "action heatmap";
        }
        ActionsHeatmap.prototype.getPosition = function (item) {
            return {
                lat: item.completed_place__location[1],
                lng: item.completed_place__location[0],
            };
        };
        return ActionsHeatmap;
    }());
    var ActionsHeatmapTrace = ExtendBoundsMixin(DataObservableMixin(ExtendBoundsMixin(HeatmapMixin(StyleMixin(MapItemsMixin(ActionsHeatmap))))));

    var __assign$2 = (undefined && undefined.__assign) || Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    var HtMapClass = /** @class */ (function () {
        // clusters = [];
        // map$ = new ReplaySubject();
        function HtMapClass(mapType, options) {
            if (mapType === void 0) { mapType = "leaflet"; }
            if (options === void 0) { options = {}; }
            this.mapType = mapType;
            // map: HtMap;
            // mapUtils: MapUtils;
            // userData$: Observable<IUserPlaceline | null>;
            this.mapInstance = new MapInstance();
            this.leafletSetBoundsOptions = {
                animate: true,
                duration: 0.3
            };
            this.googleSetBoundsOptions = {};
            this.googleMapOptions = {
                center: { lat: 0, lng: 0 },
                zoom: 2,
                fullscreenControl: false,
                streetViewControl: false,
                styles: htGoogleMapsWrapper.LightColorMapStyle
            };
            this.leafletMapOptions = { center: [3.505, 0], zoom: 2 };
            this.mapItemsSet = [];
            mapTypeService.getInstance(mapType);
            if (options.mapKey) ;
            this.usersCluster = new UsersClusterTrace(this.mapInstance);
            this.actionsCluster = new ActionsClusterTrace(this.mapInstance);
            this.usersHeatmap = new StopsHeatmapTrace(this.mapInstance);
            this.actionsHeatmap = new ActionsHeatmapTrace(this.mapInstance);
            this.placeline = new PlacelineTrace({ mapInstance: this.mapInstance });
        }
        Object.defineProperty(HtMapClass.prototype, "segmentTrace", {
            get: function () {
                return this.placeline;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(HtMapClass.prototype, "map$", {
            get: function () {
                return this.mapInstance.map$;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(HtMapClass.prototype, "map", {
            get: function () {
                return this.mapInstance.map;
            },
            enumerable: true,
            configurable: true
        });
        HtMapClass.prototype.initMap = function (elem, options) {
            if (options === void 0) { options = {}; }
            var mapOptions = this.mapType == "leaflet"
                ? this.leafletMapOptions
                : this.googleMapOptions;
            var map = this.mapInstance.mapUtils.renderMap(elem, __assign$2({}, mapOptions, options));
            this.mapInstance.setMap(map);
            return map;
        };
        // setPlacelineData$(data$: Observable<IUserPlaceline | null>) {
        //   if (this.userDataSub) {
        //     this.userDataSub.unsubscribe();
        //   }
        //   this.initUserDataObserver(data$)
        // }
        //
        // private initUserDataObserver(data$: Observable<IUserPlaceline | null>) {
        //   let userData$ = data$.scan((acc, data) => {
        //     const oldId = acc.user ? acc.user.id : null;
        //     const currentId = data ? data.id : null;
        //     const isNew = currentId && oldId ? currentId !== oldId : true;
        //     return {user: data, isNew, oldId }
        //   }, {user: null, oldId: null, isNew: true});
        //
        //   let sub = userData$.subscribe((acc) => {
        //     const userData = acc.user;
        //     const isNew = acc.isNew;
        //     this.tracePlaceline(userData);
        //     if(isNew) this.resetBounds()
        //   });
        //   this.userDataSub = sub;
        // }
        HtMapClass.prototype.tracePlaceline = function (user) {
            this.placeline.trace(user);
        };
        HtMapClass.prototype.resetBounds = function (options) {
            this.mapInstance.resetBounds(options);
        };
        HtMapClass.prototype.getBoundsItem = function (items) {
            var _this = this;
            var bounds = this.mapInstance.mapUtils.extendItemBounds();
            return items.reduce(function (bounds, item) {
                return _this.getBounds(bounds, item);
            }, bounds);
        };
        HtMapClass.prototype.getBounds = function (bounds, item) {
            return item.extendBounds(bounds);
        };
        HtMapClass.prototype.setBounds = function (bounds, options) {
            options =
                options || this.mapType == "leaflet"
                    ? this.leafletSetBoundsOptions
                    : this.googleSetBoundsOptions;
            this.mapInstance.mapUtils.setBounds(this.map, bounds, options);
        };
        HtMapClass.prototype.inValidateSize = function () {
            this.mapInstance.inValidateSize();
        };
        HtMapClass.prototype.addEntities = function (entities) {
            this.mapInstance.addToItemsSet(entities);
        };
        HtMapClass.prototype.clear = function () {
            this.mapInstance.setMap(null);
        };
        return HtMapClass;
    }());

    var MarkersBase = /** @class */ (function () {
        function MarkersBase(renderConfig, styleFunct, name) {
            this.renderConfig = renderConfig;
            this.styleFunct = styleFunct;
            this.name = name;
            this.mapInstance = renderConfig.mapInstance || new MapInstance();
        }
        MarkersBase.prototype.getPosition = function (data) {
            return this.renderConfig.getPosition(data);
        };
        MarkersBase.prototype.getInfoContent = function (data) {
            return this.renderConfig.getInfoContent(data);
        };
        return MarkersBase;
    }());

    var PolylinesBase = /** @class */ (function () {
        function PolylinesBase(renderConfig, styleFunct) {
            this.renderConfig = renderConfig;
            this.styleFunct = styleFunct;
            this.mapInstance = renderConfig.mapInstance || new MapInstance();
        }
        PolylinesBase.prototype.getEncodedPath = function (data) {
            return this.renderConfig.getEncodedPath(data);
        };
        PolylinesBase.prototype.getEncodedPositionTime = function (data) {
            return this.renderConfig.getEncodedPositionTime(data);
        };
        return PolylinesBase;
    }());

    var DivMarkersBase = /** @class */ (function () {
        function DivMarkersBase(renderConfig, styleFunct) {
            this.renderConfig = renderConfig;
            this.styleFunct = styleFunct;
            this.mapInstance = renderConfig.mapInstance || new MapInstance();
        }
        DivMarkersBase.prototype.getPosition = function (data) {
            return this.renderConfig.getPosition(data);
        };
        DivMarkersBase.prototype.getInfoContent = function (data) {
            return this.renderConfig.getInfoContent(data);
        };
        DivMarkersBase.prototype.getDivContent = function (data) {
            return this.renderConfig.getDivContent(data);
        };
        return DivMarkersBase;
    }());

    var __assign$3 = (undefined && undefined.__assign) || Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    var mapItemsFactory = function (baseClass, config) {
        var defaultConfig = {
            isCluster: false,
            hasPopup: false,
            isPolyline: false,
            isDiv: false,
            isSingleItem: false,
            isCircle: false,
            hasDataObservable: false
        };
        var finalConfig = __assign$3({}, defaultConfig, config);
        var itemClass = baseClass;
        itemClass = MapItemsMixin(itemClass);
        itemClass = MarkersMixin(itemClass);
        itemClass = StyleMixin(itemClass);
        itemClass = TraceMixin(itemClass);
        itemClass = ExtendBoundsMixin(itemClass);
        if (finalConfig.isSingleItem)
            itemClass = SingleItemMixin(itemClass);
        if (finalConfig.hasPopup)
            itemClass = PopupMixin(itemClass);
        if (finalConfig.isCluster)
            itemClass = ClusterMixin(itemClass);
        if (finalConfig.isDiv)
            itemClass = DivMarkersMixin(itemClass);
        if (finalConfig.isCircle)
            itemClass = CircleMixin(itemClass);
        if (finalConfig.isPolyline)
            itemClass = PolylinesMixin(itemClass);
        if (finalConfig.hasDataObservable)
            itemClass = DataObservableMixin(itemClass);
        return itemClass;
    };
    var itemsBaseFactory = function (_a) {
        var renderConfig = _a.renderConfig, typeConfig = _a.typeConfig, styleFunct = _a.styleFunct;
        var mapTypesBase = {
            polylines: PolylinesBase,
            markers: MarkersBase,
            divMarkers: DivMarkersBase
        };
        typeConfig = typeConfig || {};
        var MapItemBase = mapTypesBase.markers;
        if (typeConfig.isDiv)
            MapItemBase = mapTypesBase.divMarkers;
        if (typeConfig.isPolyline)
            MapItemBase = mapTypesBase.polylines;
        var base = mapItemsFactory(MapItemBase, typeConfig);
        return base;
    };
    var itemsFactory = function (_a) {
        var renderConfig = _a.renderConfig, typeConfig = _a.typeConfig, styleFunct = _a.styleFunct, styleObj = _a.styleObj;
        var base = itemsBaseFactory({ renderConfig: renderConfig, typeConfig: typeConfig, styleFunct: styleFunct, styleObj: styleObj });
        function getStyleFunct(styleObj) {
            return {
                get: function (type) {
                    return styleObj[type];
                }
            };
        }
        return new base(renderConfig, styleFunct || getStyleFunct(styleObj));
    };

    var DestinationMap = /** @class */ (function () {
        function DestinationMap(mapInstance) {
            this.name = "Destination";
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case 'google': {
                            return {
                                default: {},
                                popup: {
                                    pixelOffset: new google.maps.Size(0, -5),
                                }
                            };
                        }
                        case 'leaflet': {
                            return {
                                default: {
                                    zIndexOffset: 10,
                                    iconSize: [30, 30],
                                    className: "destination-marker"
                                }
                            };
                        }
                    }
                }
            };
            this.mapInstance = mapInstance;
        }
        DestinationMap.prototype.getPosition = function (data) {
            var place = data.completed_place || data.expected_place;
            if (place && place.location) {
                return { lat: place.location.coordinates[1], lng: place.location.coordinates[0] };
            }
            else {
                return null;
            }
        };
        DestinationMap.prototype.getDivContent = function () {
            return "<div style=\"width: 30px; height: 30px; background: #00C94B; border-radius: 50%\"></div>";
        };
        return DestinationMap;
    }());
    var DestinationMarker = DataObservableMixin(ExtendBoundsMixin(TraceMixin(DivMarkersMixin(MarkersMixin(StyleMixin(MapItemsMixin(DestinationMap)))))));

    var __extends$17 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    var ReplayMarker = /** @class */ (function (_super) {
        __extends$17(ReplayMarker, _super);
        function ReplayMarker() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.name = "replay marker";
            _this.styleObj = {
                default: {
                    radius: 6,
                    fillColor: htUtility.Color.greenDark,
                    fillOpacity: 1,
                    weight: 2,
                    color: 'black',
                    pane: "markerPane"
                },
                popup: {
                    offset: [0, -5],
                    // offset: point(0, -5),
                    closeButton: false
                },
                highlight: {
                    radius: 10,
                    fillColor: htUtility.Color.stop,
                    fillOpacity: 1,
                    weight: 4,
                    color: htUtility.Color.stopDark,
                    pane: "markerPane"
                },
                fade: {
                    radius: 10,
                    fillColor: htUtility.Color.stop,
                    fillOpacity: 0.1,
                    weight: 2,
                    color: htUtility.Color.stopDark,
                    pane: "markerPane"
                }
            };
            return _this;
        }
        ReplayMarker.prototype.getPosition = function (data) {
            return data.currentPosition;
        };
        ReplayMarker.prototype.getBearing = function (data) {
            return data.bearing;
        };
        ReplayMarker.prototype.getInfoContent = function (data) {
            var time = "<div class=\"has-text-weight-bold\">" + htUtility.TimeString(data.currentTime) + "</div>";
            var activity = data.currentSegment ? "<div>" + htUtility.NameCase(data.currentSegment.type) + "</div>" : "";
            return "<div class=\"flex-column justify-center text-center\">" + time + activity + "</div>";
        };
        return ReplayMarker;
    }(CurrentUser));
    var ReplayMarkerTrace = SingleItemMixin(CircleMixin(PopupMixin(TraceMixin(MarkersMixin(StyleMixin(MapItemsMixin(ReplayMarker)))))));
    // import {SetMap} from "../../utils/map-utls";
    // import {Color} from "../../utils/color";
    // import {HtMarkerItem} from "./ht-js-map/marker-item";
    // import {divIcon, marker} from "leaflet";
    //
    //
    // export class ReplayMarker extends HtMarkerItem {
    //     item: L.Marker = marker([0,0], {
    //         pane: 'popupPane',
    //         zIndexOffset: 110
    //     });
    //     test = marker([0,0]);
    //
    //     setPositionBearing(position, bearing, map) {
    //         this.setIcon(bearing);
    //         this.item.setLatLng(position);
    //         SetMap(this.item, map)
    //     }
    //
    //     private getIcon(bearing: number = 0): L.DivIcon {
    //         return divIcon({
    //             html: this.getDivContent(bearing),
    //             className: 'current-user-marker',
    //             iconSize: [50, 50]
    //         })
    //     }
    //
    //     private setIcon(bearing: number = 0) {
    //         this.item.setIcon(this.getIcon(bearing))
    //     }
    //
    //     private getDivContent(bearing: number): string {
    //         let icon = `<div id="user-marker" style="opacity: 0.7">
    // <div class="marker">  <svg style="transform: rotateZ(${bearing}deg) scale(0.83)" fill="${Color.blueDark}" viewBox="0 0 96 96"><g><path style="; stroke-width: 4px" d="M51.58,19.16l27.15,54.3a4,4,0,0,1-5.33,5.39l-24.6-12a4,4,0,0,0-3.58,0L22.88,78.41a4,4,0,0,1-5.41-5.35l27-53.9A4,4,0,0,1,51.58,19.16Z"/></g></svg></div>
    // </div>`;
    //         return icon
    //     }
    // }

    var __extends$18 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    function AnimationsEntitiesMixin(Base) {
        return /** @class */ (function (_super) {
            __extends$18(class_1, _super);
            function class_1() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                // bearing: number = 0;
                // item;
                _this.isAnimated = true;
                _this.toNotTraceItem = false;
                _this.subs = {};
                return _this;
            }
            class_1.prototype.setTimeAwareAnimationEntity = function (animationEntity) {
                this.animationEntities = animationEntity || new AnimationsEntities();
            };
            class_1.prototype.clearItem = function (entity) {
                var id = this.trackBy(entity.data);
                var animId = entity.data.id;
                this.animationEntities.clearItem(animId);
                this.clearSub(id);
                _super.prototype.clearItem.call(this, entity);
            };
            class_1.prototype.update = function (entity, pathBearing) {
                var id = this.trackBy(entity.data);
                var animId = entity.data.id;
                this.initSub(id, animId);
                var encodedString = this.getTimeAwarePolyline ? this.getTimeAwarePolyline(entity.data) : null;
                var timeAwarePath = this.getTimeAwarePath ? this.getTimeAwarePath(entity.data) : null;
                if (timeAwarePath) {
                    this.animationEntities.updatePath(id, timeAwarePath);
                }
                else if (encodedString) {
                    this.animationEntities.update(id, encodedString);
                }
            };
            class_1.prototype.initSub = function (id, animId) {
                var _this = this;
                animId = animId || id;
                var sub = this.subs[id];
                if (sub) ;
                else {
                    var newsub = this.animationEntities
                        .getEntity(animId)
                        .updateEvent
                        .subscribe('update', function (_a) {
                        var path = _a.path, bearing = _a.bearing;
                        var entity = _this.getEntity(id);
                        if (entity) {
                            _super.prototype.update.call(_this, entity, { path: path, bearing: bearing });
                        }
                    });
                    this.subs[id] = newsub;
                }
            };
            class_1.prototype.clearSub = function (id) {
                var sub = this.subs[id];
                sub.unsubscribe();
                delete this.subs[id];
            };
            return class_1;
        }(Base));
    }
    var AnimationsEntities = /** @class */ (function () {
        function AnimationsEntities() {
            this.enitites = {};
        }
        AnimationsEntities.prototype.clearItem = function (id) {
            var entity = this.enitites[id];
            entity.clear();
            delete this.enitites[id];
        };
        AnimationsEntities.prototype.update = function (id, polyline) {
            var entity = this.getEntity(id);
            entity.updatePolylineString(polyline);
        };
        AnimationsEntities.prototype.updatePath = function (id, timeAwarePath) {
            var entity = this.getEntity(id);
            entity.update(timeAwarePath);
        };
        AnimationsEntities.prototype.getEntity = function (id) {
            var entity = this.enitites[id];
            if (entity) {
                return entity;
            }
            else {
                var newEntity = new timeAwarePolyline.TimeAwareAnimation();
                this.enitites[id] = newEntity;
                return newEntity;
            }
        };
        return AnimationsEntities;
    }());

    var ActionsDataPolyline = /** @class */ (function () {
        function ActionsDataPolyline(mapInstance) {
            this.name = "Actions data polyline";
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case 'google': {
                            return {
                                default: {},
                                popup: {
                                    pixelOffset: new google.maps.Size(0, -5),
                                }
                            };
                        }
                        case 'leaflet': {
                            return {
                                default: {
                                    fillColor: '#9155f4',
                                    color: '#9155f4',
                                    weight: 5
                                    // iconAnchor: [12, 12]
                                    // iconSize: [35, 35],
                                    // className: 'current-action-marker',
                                    // iconAnchor: point(15, 43)
                                    // iconAnchor: [15, 43]
                                },
                                popup: {
                                    // offset: point(0, -35),
                                    offset: [0, -5],
                                    closeButton: false
                                }
                            };
                        }
                    }
                }
            };
            this.mapInstance = mapInstance;
        }
        ActionsDataPolyline.prototype.isValidMapItems = function (data) {
            return !!this.getTimeAwarePath(data);
        };
        ActionsDataPolyline.prototype.getTimeAwarePath = function (data) {
            return data.timeAwarePath;
        };
        ActionsDataPolyline.prototype.getEncodedPositionTime = function (data) {
            return data.location_time_series;
        };
        //todo remove this, use getTimeAwarePolyline
        ActionsDataPolyline.prototype.getEncodedPath = function (data) {
            return data.route;
        };
        ActionsDataPolyline.prototype.getPosition = function (action) {
            var lastLocation = action.timeAwarePath ? action.timeAwarePath[action.timeAwarePath.length - 1] : null;
            return lastLocation ? { lat: +lastLocation[0], lng: +lastLocation[1] } : null;
        };
        ActionsDataPolyline.prototype.getTimeAwarePolyline = function (data) {
            return data.location_time_series;
        };
        return ActionsDataPolyline;
    }());
    // export interface IActionsPolylineBase {
    //   mapInstance: MapInstance
    //   // getStyle: (styleType?) => object;
    // }
    //
    // export function ActionsPolylineMixin<TBase extends Constructor<IActionsPolylineBase>>(Base: TBase) {
    //   return class extends Base {
    //     start;
    //
    //
    //
    //
    //
    //   };
    // }
    var ActionsPolylineTrace$1 = DataObservableMixin(AnimationsEntitiesMixin(TraceMixin(ExtendBoundsMixin(PolylinesMixin(MarkersMixin(StyleMixin(MapItemsMixin(ActionsDataPolyline))))))));
    // export const ActionsPolylineTrace = DataObservableMixin(AnimationMixin(TraceMixin(PolylinesMixin(StyleMixin(ActionsDataPolyline)))))

    var StartMarker = /** @class */ (function () {
        function StartMarker(mapInstance) {
            this.name = "Start marker";
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case 'google': {
                            return {
                                default: {},
                                popup: {
                                    pixelOffset: new google.maps.Size(0, -5),
                                }
                            };
                        }
                        case 'leaflet': {
                            return {
                                default: {
                                    zIndexOffset: 10,
                                    iconSize: [20, 20],
                                    className: "destination-marker"
                                }
                            };
                        }
                    }
                }
            };
            this.mapInstance = mapInstance;
        }
        StartMarker.prototype.trace = function (mapItem) {
        };
        StartMarker.prototype.getPosition = function (data) {
            var place = data.started_place;
            if (place && place.location) {
                return { lat: place.location.coordinates[1], lng: place.location.coordinates[0] };
            }
            else {
                return null;
            }
        };
        StartMarker.prototype.getDivContent = function () {
            return "<div style=\"width: 20px; height: 20px; background: #9013FE; border-radius: 50%\"></div>";
        };
        return StartMarker;
    }());
    var StartMarkerTrace = ExtendBoundsMixin(DataObservableMixin(TraceMixin(DivMarkersMixin(MarkersMixin(StyleMixin(MapItemsMixin(StartMarker)))))));

    var ActionUser = /** @class */ (function () {
        function ActionUser(mapInstance) {
            this.mapInstance = mapInstance;
            this.name = "action user";
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case "google":
                            {
                                return {
                                    default: {
                                        flat: true,
                                        anchor: RichMarkerPosition.BOTTOM_CENTER
                                    }
                                };
                            }

                        case "leaflet": {
                            return {
                                default: {
                                    zIndexOffset: 10,
                                    iconSize: [60, 60],
                                    className: "action-marker"
                                    // iconAnchor: point(15, 43)
                                    // iconAnchor: [0, 0]
                                }
                            };
                        }
                    }
                }
            };
        }
        ActionUser.prototype.getPosition = function (data) {
            var lastLocation = data.location;
            var lastPosition = lastLocation ? lastLocation.geojson.coordinates : null;
            return lastPosition ? { lat: lastPosition[1], lng: lastPosition[0] } : null;
        };
        ActionUser.prototype.trackBy = function (action) {
            return action.user.id;
        };
        ActionUser.prototype.trackAnimationBy = function (action) {
            return action.id;
        };
        ActionUser.prototype.getDivContent = function (data) {
            var content = "\n    <div style=\"border-radius: 50%; height: 60px; width: 60px; background: rgba(95,143,213,0.67) \">\n  <div style=\"height: 30px; width: 30px; background-image: url('" + data.user.photo + "'); background-repeat: no-repeat;\n  background-size: cover;     top: 15px;\n    position: relative;\n    left: 15px;\n    border-radius: 50%;\"></div>\n</div> \n    ";
            return content;
        };
        return ActionUser;
    }());
    var ActionUserTrace = DataObservableMixin(ExtendBoundsMixin(AnimationsEntitiesMixin(DivMarkersMixin(TraceMixin(MarkersMixin(StyleMixin(MapItemsMixin(ActionUser))))))));

    var ActionsExpectedPolyline = /** @class */ (function () {
        function ActionsExpectedPolyline(mapInstance) {
            this.mapInstance = mapInstance;
            this.lineSymbol = {
                path: 'M 0,-1 0,1',
                strokeOpacity: 1,
                scale: 4
            };
            this.styleFunct = {
                get: function (type) {
                    switch (type) {
                        case "google":
                            {
                                return {
                                    default: {
                                        strokeColor: htUtility.Color.grey3,
                                        strokeOpacity: 0,
                                        strokeWeight: 5,
                                        icons: [{
                                                icon: {
                                                    path: 'M 0,-1 0,0',
                                                    strokeOpacity: 1,
                                                    scale: 4
                                                },
                                                offset: '0',
                                                repeat: '13px'
                                            }],
                                    },
                                    highlight: {
                                        strokeColor: htUtility.Color.grey2,
                                        strokeOpacity: 1,
                                        strokeWeight: 5
                                    },
                                    fade: {
                                        strokeColor: htUtility.Color.grey2,
                                        strokeOpacity: 0.2,
                                        strokeWeight: 2
                                    }
                                };
                            }

                        case "leaflet": {
                            return {
                                default: {
                                    weight: 5,
                                    color: htUtility.Color.grey4,
                                    opacity: 1,
                                    dashArray: "7 10"
                                },
                                highlight: {
                                    weight: 5,
                                    color: htUtility.Color.grey2,
                                    opacity: 1
                                },
                                fade: {
                                    weight: 2,
                                    color: htUtility.Color.grey2,
                                    opacity: 0.2
                                }
                            };
                        }
                    }
                },
            };
            this.name = "action expected polyline";
        }
        ActionsExpectedPolyline.prototype.isValidMapItems = function (data) {
            return !!this.getEncodedPath(data);
        };
        ActionsExpectedPolyline.prototype.getEncodedPath = function (data) {
            return data.expected_route;
        };
        ActionsExpectedPolyline.prototype.getPosition = function () {
            return { lat: 0, lng: 0 };
        };
        return ActionsExpectedPolyline;
    }());
    var ActionsExpectedPolylineTrace = DataObservableMixin(TraceMixin(ExtendBoundsMixin(PolylinesMixin(MarkersMixin(StyleMixin(MapItemsMixin(ActionsExpectedPolyline)))))));

    var ActionMap = /** @class */ (function () {
        function ActionMap(mapInstance, options) {
            if (options === void 0) { options = {}; }
            this.anim = new AnimationsEntities();
            this.mapInstance = mapInstance;
            this.destination = new DestinationMarker(mapInstance);
            this.polyline = new ActionsPolylineTrace$1(mapInstance);
            this.polyline.setTimeAwareAnimationEntity(this.anim);
            this.user = new ActionUserTrace(mapInstance);
            this.user.setTimeAwareAnimationEntity(this.anim);
            if (options.hasPulse) {
                this.pulse = new ActionUserTrace(mapInstance);
                this.pulse.setTimeAwareAnimationEntity(this.anim);
            }
            if (!options.hideExpectedPolyline) {
                this.expectedPolyline = new ActionsExpectedPolylineTrace(this.mapInstance);
            }
            // this.user.setTimeAwareAnimation(this.anim);
            // this.user.toNotTraceItem = true;
            this.start = new StartMarkerTrace(mapInstance);
        }
        ActionMap.prototype.setData$ = function (data$) {
            this.destination.setData$(data$);
            this.polyline.setData$(data$);
            this.start.setData$(data$);
            var users$ = data$.pipe(operators.map(function (actions) {
                return actions.reduce(function (acc, action) {
                    return action.completed_at ? acc : acc.concat([action]);
                }, []);
            }));
            if (this.pulse)
                this.pulse.setData$(users$);
            if (this.expectedPolyline)
                this.expectedPolyline.setData$(users$);
            this.user.setData$(users$);
        };
        return ActionMap;
    }());
    var ActionTrace = ActionMap;

    var __extends$19 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    var __assign$4 = (undefined && undefined.__assign) || Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    var TimelineReplay = /** @class */ (function (_super) {
        __extends$19(TimelineReplay, _super);
        function TimelineReplay() {
            var _this = _super.call(this) || this;
            _this.stats$ = new rxjs.BehaviorSubject(null);
            _this.head$ = new rxjs.BehaviorSubject(null);
            _this.player$ = new rxjs.BehaviorSubject({ isPlaying: false, isStopped: true, speed: 2 });
            _this.debug = false;
            _this.frameInterval = 50;
            _this.skipStops = false;
            _this.addListerner();
            return _this;
        }
        TimelineReplay.prototype.addListerner = function () {
            var _this = this;
            this.stats$.subscribe(function (stats) {
                _this.stats = stats;
            });
            this.head$.subscribe(function (head) {
                _this.head = head;
            });
            this.player$.subscribe(function (player) {
                _this.player = player;
                if (player.isStopped)
                    _this.currentSegmentEffects(null);
            });
        };
        TimelineReplay.prototype.getPositionBearingnAtTime = function (time) {
            if (!this.stats)
                return null;
            // let currentTimeValue = (timePercent * (this.stats.duration) / 100) + new Date(this.stats.start).getTime();
            // let time = new Date(currentTimeValue).toISOString();
            // console.log(TimeString(time));
            // if(this.player && !this.player.isStopped)
            // if(segment) console.log(segment.type, "segment", TimeString(segment.started_at), TimeString(segment.ended_at));
            var position;
            var bearing;
            var pathSegment = this.getLocationsTillTimeStamp(this.timeAwareArray, time);
            if (pathSegment && pathSegment.locations.length > 0) {
                // let pathBeaing = _.last(pathSegment);
                var point = _.last(pathSegment.locations);
                position = { lat: point[0], lng: point[1] };
                bearing = pathSegment.bearing;
                // return [point[0], point[1]]
            }
            return { position: position, bearing: bearing };
        };
        TimelineReplay.prototype.setStats = function (stats) {
            this.stats$.next(stats);
        };
        TimelineReplay.prototype.setReplayHead = function (head) {
            this.head$.next(head);
        };
        TimelineReplay.prototype.getReplayStats = function () {
            return this.stats$.pipe(operators.share());
        };
        TimelineReplay.prototype.getReplayHead = function () {
            return this.head$.pipe(operators.share());
        };
        TimelineReplay.prototype.currentTimeEffects = function (time) {
        };
        TimelineReplay.prototype.currentSegmentEffects = function (segment) {
        };
        TimelineReplay.prototype.getLastPositionBearing = function () {
            var lastSegTime = _.last(this.timeAwareArray)[2] + '';
            return this.getPositionBearingnAtTime(lastSegTime);
        };
        TimelineReplay.prototype.getLocationsAtTimesT = function (times) {
            return this.getLocationsAtTimestamps(this.timeAwareArray, times);
        };
        // getBounds(bounds: L.LatLngBounds) {
        //   return _.reduce(this.timeAwareArray, (bounds, point) => {
        //     return bounds.extend([+point[0], +point[1]])
        //   }, bounds)
        // }
        TimelineReplay.prototype.getNoTrackingSegments = function (events) {
            return events.reduce(function (acc, event) {
                if (acc.start && event.type == 'tracking.started') {
                    return {
                        segments: acc.segments.concat([[acc.start, event.recorded_at]]),
                        start: ''
                    };
                }
                else if (event.type == 'tracking.ended') {
                    return __assign$4({}, acc, { start: event.recorded_at });
                }
                return acc;
            }, {
                segments: [],
                start: ""
            }).segments;
        };
        //replay player
        TimelineReplay.prototype.goToTimePercent = function (timePercent, toPause) {
            if (toPause === void 0) { toPause = false; }
            if (timePercent < 0)
                timePercent = 0;
            if (timePercent > 100)
                timePercent = 100;
            var time = this.getTimeFromTimePercent(timePercent);
            if (toPause) {
                if (this.player.isStopped)
                    this.setPlayer({ isStopped: false });
                this.jumpToTime(time, timePercent);
            }
            else {
                this.goToTime(time, timePercent);
            }
        };
        TimelineReplay.prototype.jumpToTimePercent = function (timePercent) {
            this.goToTimePercent(timePercent, true);
        };
        TimelineReplay.prototype.getNextTimePercent = function (head) {
            return head ? head.timePercent + this.getIncTimePercent(head) : 0;
        };
        TimelineReplay.prototype.getIncTimePercent = function (head) {
            var normalSpeed = 6000;
            var duration = normalSpeed / this.player.speed;
            var segment = head.currentSegment;
            if (segment) {
                if (segment.type == 'trip') {
                    var max = 2 * 60 * 60 * 1000;
                    duration = (normalSpeed * Math.min(max, head.currentSegment.durationSeg) / max + normalSpeed) / this.player.speed;
                }
                var frameStep = duration / this.frameInterval;
                var segmentGap = (segment.endPercent - segment.startPercent);
                var segmentCurrentGap = segment.endPercent - head.timePercent;
                var maxInc = segmentCurrentGap > 0 ? Math.min(segmentGap, segmentCurrentGap) : segmentGap;
                return Math.min(segmentGap / frameStep, maxInc);
            }
            else {
                this.pause();
                return 0;
            }
        };
        TimelineReplay.prototype.getTimeFromTimePercent = function (timePercent) {
            var currentTimeValue = (timePercent * (this.stats.duration) / 100) + new Date(this.stats.start).getTime();
            var currentTime = new Date(currentTimeValue).toISOString();
            return currentTime;
        };
        TimelineReplay.prototype.jumpToTime = function (time, timePercent) {
            this.pause();
            this.goToTime(time, timePercent);
        };
        TimelineReplay.prototype.goToTime = function (time, timePercent) {
            //get head and update head$
        };
        TimelineReplay.prototype.clear = function () {
            if (!this.player.isStopped)
                this.stop();
            this.timeAwareArray = null;
            this.setStats(null);
        };
        TimelineReplay.prototype.play = function () {
            var _this = this;
            this.setPlayer({ isStopped: false, isPlaying: true });
            var takeUtil$ = this.player$.pipe(operators.filter(function (player) { return !player.isPlaying; }), operators.take(1));
            this.playerSub = rxjs.timer(0, this.frameInterval).pipe(operators.switchMap(function () { return _this.head$.pipe(operators.take(1)); }), operators.map(function (head) { return _this.getNextTimePercent(head); }), operators.takeUntil(takeUtil$))
                .subscribe(function (timePercent) {
                _this.goToTimePercent(timePercent);
            });
        };
        TimelineReplay.prototype.toggleSkipStops = function () {
            this.skipStops = !this.skipStops;
        };
        TimelineReplay.prototype.pause = function () {
            this.setPlayer({ isPlaying: false });
        };
        TimelineReplay.prototype.stop = function () {
            this.jumpToTimePercent(0);
            this.setPlayer({ isStopped: true, isPlaying: false, speed: 2 });
            this.setReplayHead(null);
        };
        TimelineReplay.prototype.setSpeed = function (speed) {
            this.setPlayer({ speed: speed });
        };
        TimelineReplay.prototype.setPlayer = function (obj) {
            this.player$.next(__assign$4({}, this.player, obj));
        };
        return TimelineReplay;
    }(timeAwarePolyline.TimeAwareEncoder));

    var __extends$20 = (undefined && undefined.__extends) || (function () {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    var __assign$5 = (undefined && undefined.__assign) || Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    var TimelineSegment = /** @class */ (function (_super) {
        __extends$20(TimelineSegment, _super);
        function TimelineSegment() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        TimelineSegment.prototype.update = function (userData) {
            var _this = this;
            var segments = userData.placeline;
            // let noTrackingSegments = this.getNoTrackingSegments(userData.events);
            // console.log(noTrackingSegments);
            var lastUpdated = userData.last_heartbeat_at;
            lastUpdated = lastUpdated || new Date().toISOString();
            var duration = 0;
            var totalTimeAwareArray = [];
            this.allSegments = _.reduce(segments, function (acc, segment, i) {
                var segmentLastUpdatedAt = i == segments.length - 1 ? lastUpdated : null;
                var segmentData = _this.getSegmentData(segment, segmentLastUpdatedAt);
                var currentSegment;
                var gapSegment;
                if (segmentData) {
                    duration = duration + segmentData.durationSeg;
                    totalTimeAwareArray = totalTimeAwareArray.concat(segmentData.timeAwareArray);
                    currentSegment = __assign$5({}, segment, segmentData);
                }
                gapSegment = acc.length && currentSegment ? _this.getGapSegment(currentSegment, _.last(acc)) : null;
                if (gapSegment) {
                    var gapSegmentData = _this.getSegmentData(gapSegment, segmentLastUpdatedAt);
                    duration = duration + gapSegmentData.durationSeg;
                    gapSegment = __assign$5({}, gapSegment, gapSegmentData);
                }
                acc = gapSegment ? acc.concat([gapSegment]) : acc;
                acc = currentSegment ? acc.concat([currentSegment]) : acc;
                return acc;
            }, []);
            this.timeAwareArray = totalTimeAwareArray;
            // this.timeAwareArray = _.sortBy(totalTimeAwareArray, (array => {
            //   let time = new Date(array[2]).getTime();
            //   return Math.floor(time / 60000)
            // }));
            //sorting messes up stop end and trip start points.
            this.duration = duration;
            this.segments = this.getSegmentsWithPercentMarks(this.allSegments, duration);
            var stats = this.getStats(this.segments);
            this.setStats(stats);
            // console.log(this.segments, "deco");
        };
        TimelineSegment.prototype.currentTimeEffects = function (time) {
        };
        TimelineSegment.prototype.currentSegmentEffects = function (currentSegment) {
            if (this.playSegmentCallback) {
                var segment = currentSegment;
                var segmentId = segment ? segment.id : '';
                this.playSegmentCallback(segmentId);
            }
        };
        TimelineSegment.prototype.getCurrentSegment = function (time) {
            var timeStamp = new Date(time).getTime();
            return this.segments.find(function (segment) {
                return segment.start <= timeStamp && segment.end > timeStamp;
            });
        };
        TimelineSegment.prototype.getSegmentsWithPercentMarks = function (segments, duration) {
            segments = segments.reduce(function (acc, segment) {
                var startPercent = acc.pStart;
                var segPercent = (segment.durationSeg / duration) * 100;
                var endPercent = Math.min(startPercent + segPercent, 100);
                var filledSegment = __assign$5({}, segment, { startPercent: startPercent,
                    endPercent: endPercent });
                return {
                    segments: acc.segments.concat([filledSegment]),
                    pStart: endPercent,
                    bearing: 0
                };
            }, { segments: [], pStart: 0, bearing: 0 }).segments;
            return segments;
        };
        TimelineSegment.prototype.getTripTimeAwareArray = function (segment, segmentEnd) {
            var timeAwareArray = [];
            // if(segment.start_location && segment.start_location.geojson && segment.started_at) {
            //   let startCord = segment.start_location.geojson.coordinates;
            //   timeAwareArray.push([startCord[1], startCord[0], segment.started_at])
            // }
            if (segment.location_time_series) {
                timeAwareArray.push.apply(timeAwareArray, this.decodeTimeAwarePolyline(segment.location_time_series));
            }
            else if (segmentEnd) {
                var lastPoint = _.last(timeAwareArray);
                if (lastPoint)
                    timeAwareArray.push([lastPoint[0], lastPoint[1], segmentEnd]);
            }
            return timeAwareArray;
        };
        TimelineSegment.prototype.getStopTimeAwareArray = function (segment, segmentEnd) {
            var timeAwareArray = [];
            var coord = segment.place && segment.place.location ? segment.place.location.coordinates : null;
            if (coord) {
                var position = [coord[1], coord[0]];
                timeAwareArray = [position.concat([segment.started_at]), position.concat([segmentEnd])];
            }
            return timeAwareArray;
        };
        TimelineSegment.prototype.getStats = function (segments) {
            if (segments.length) {
                var lastSeg = _.last(segments);
                var start = segments[0].start;
                var end = lastSeg.end;
                //todo duration as sum of segments
                var stats = {
                    start: new Date(start).toISOString(),
                    end: new Date(end).toISOString(),
                    duration: end - start,
                    distance: 0,
                    timeAwarePolylineArray: this.timeAwareArray,
                    placeline: this.segments
                };
                return stats;
            }
            return null;
        };
        TimelineSegment.prototype.getSegmentData = function (segment, lastUpdatedAt) {
            var segmentEnd = segment.ended_at || lastUpdatedAt;
            var segmentStart = segment.started_at;
            if (!segmentEnd || !segmentStart)
                return null;
            var end = new Date(segmentEnd).getTime();
            var start = new Date(segmentStart).getTime();
            var durationSeg = end - start;
            var timeAwareArray;
            if (segment && segment.type !== 'stop' && segment.type !== 'unknown' && segment.type != 'gap') {
                timeAwareArray = this.getTripTimeAwareArray(segment, segmentEnd);
            }
            else if (!segment.timeAwareArray) {
                timeAwareArray = this.getStopTimeAwareArray(segment, segmentEnd);
            }
            return {
                timeAwareArray: timeAwareArray,
                end: end,
                start: start,
                durationSeg: durationSeg
            };
        };
        TimelineSegment.prototype.getGapSegment = function (segment, segment2) {
            // let firstPoint = ;
            // let lastPoint = _.first(segment.timeAwareArray);
            var started_at = segment2.ended_at;
            var ended_at = segment.started_at;
            var timeAwareArray = [];
            // console.log(firstPoint, lastPoint, "fl");
            if (started_at && ended_at) {
                // let started_at = firstPoint[2];
                // let ended_at =  lastPoint[2];
                return segment.start - segment2.end > 3 * 60000 ?
                    {
                        ended_at: ended_at,
                        started_at: started_at,
                        timeAwareArray: timeAwareArray,
                        type: 'gap'
                    } : null;
            }
            else {
                return null;
            }
        };
        TimelineSegment.prototype.clearTimeline = function () {
            this.clear();
        };
        TimelineSegment.prototype.goToTime = function (time, timePercent) {
            //get head and update head$
            var _a = this.getPositionBearingnAtTime(time), position = _a.position, bearing = _a.bearing;
            var currentSegment = this.getCurrentSegment(time);
            var head = {
                currentTime: time,
                timePercent: timePercent,
                currentPosition: position,
                bearing: bearing,
                currentSegment: currentSegment,
                segmentPercent: 0
            };
            this.currentSegmentEffects(currentSegment);
            this.currentTimeEffects(time);
            this.setReplayHead(head);
        };
        return TimelineSegment;
    }(TimelineReplay));

    var ReplayTrace = /** @class */ (function () {
        function ReplayTrace() {
            this.timelineSegment = new TimelineSegment();
        }
        ReplayTrace.prototype.getSegmentTypes = function (userSegments) {
            return userSegments.reduce(function (segmentType, segment) {
                if (segment.type == 'stop') {
                    if (segment.place && segment.place.location)
                        segmentType.stopSegment.push(segment);
                }
                else {
                    if (segment.route)
                        segmentType.tripSegment.push(segment);
                }
                return segmentType;
            }, { tripSegment: [], stopSegment: [] });
        };
        //segments replay
        ReplayTrace.prototype.clearTimeline = function () {
            this.timelineSegment.clearTimeline();
        };
        Object.defineProperty(ReplayTrace.prototype, "stats", {
            get: function () {
                return this.timelineSegment.stats;
            },
            enumerable: true,
            configurable: true
        });
        // get segments() {
        //   return this.timelineSegment.placeline
        // }
        ReplayTrace.prototype.updateTimeline = function (user) {
            this.timelineSegment.update(user);
        };
        ReplayTrace.prototype.setSegmentPlayCallback = function (cb) {
            this.timelineSegment.playSegmentCallback = cb;
        };
        return ReplayTrace;
    }());

    exports.MapInstance = MapInstance;
    exports.Placeline = Placeline;
    exports.PlacelineTrace = PlacelineTrace;
    exports.HtMapClass = HtMapClass;
    exports.mapTypeService = mapTypeService;
    exports.mapItemsFactory = mapItemsFactory;
    exports.itemsBaseFactory = itemsBaseFactory;
    exports.itemsFactory = itemsFactory;
    exports.MarkersBase = MarkersBase;
    exports.UsersCluster = UsersCluster;
    exports.UsersClusterTrace = UsersClusterTrace;
    exports.ActionMarkers = ActionMarkers;
    exports.ActionMarkersTrace = ActionMarkersTrace;
    exports.SegmentPolylines = SegmentPolylines;
    exports.SegmentPolylinesTrace = SegmentPolylinesTrace;
    exports.StopMarkers = StopMarkers;
    exports.StopMarkersTrace = StopMarkersTrace;
    exports.CurrentUser = CurrentUser;
    exports.CurrentUserTrace = CurrentUserTrace;
    exports.ActionsHeatmap = ActionsHeatmap;
    exports.ActionsHeatmapTrace = ActionsHeatmapTrace;
    exports.StopsHeatmap = StopsHeatmap;
    exports.StopsHeatmapTrace = StopsHeatmapTrace;
    exports.ActionsCluster = ActionsCluster;
    exports.ActionsClusterTrace = ActionsClusterTrace;
    exports.DestinationMap = DestinationMap;
    exports.DestinationMarker = DestinationMarker;
    exports.ReplayMarker = ReplayMarker;
    exports.ReplayMarkerTrace = ReplayMarkerTrace;
    exports.CompoundDataObservableMixin = CompoundDataObservableMixin;
    exports.AnimationMixin = AnimationMixin;
    exports.CircleMixin = CircleMixin;
    exports.ClusterMixin = ClusterMixin;
    exports.DataObservableMixin = DataObservableMixin;
    exports.DivMarkersMixin = DivMarkersMixin;
    exports.ExtendBoundsMixin = ExtendBoundsMixin;
    exports.HeatmapMixin = HeatmapMixin;
    exports.MarkersMixin = MarkersMixin;
    exports.PolylinesMixin = PolylinesMixin;
    exports.PopupMixin = PopupMixin;
    exports.SingleItemMixin = SingleItemMixin;
    exports.StyleMixin = StyleMixin;
    exports.TraceMixin = TraceMixin;
    exports.MapItemsMixin = MapItemsMixin;
    exports.ActionMap = ActionMap;
    exports.ActionTrace = ActionTrace;
    exports.ReplayTrace = ReplayTrace;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
