"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var marker_renderer_1 = require("../mixins/marker-renderer");
var polyline_renderer_1 = require("../mixins/polyline-renderer");
var extend_bounds_1 = require("../mixins/extend-bounds");
var trace_1 = require("../mixins/trace");
var styles_1 = require("../mixins/styles");
var ht_utility_1 = require("ht-utility");
var map_items_1 = require("../mixins/map-items");
var data_observable_1 = require("../mixins/data-observable");
var ActionsExpectedPolyline = /** @class */ (function () {
    function ActionsExpectedPolyline(mapInstance) {
        this.mapInstance = mapInstance;
        this.lineSymbol = {
            path: 'M 0,-1 0,1',
            strokeOpacity: 1,
            scale: 4
        };
        this.styleFunct = {
            get: function (type) {
                switch (type) {
                    case "google":
                        {
                            return {
                                default: {
                                    strokeColor: ht_utility_1.Color.grey3,
                                    strokeOpacity: 0,
                                    strokeWeight: 5,
                                    icons: [{
                                            icon: {
                                                path: 'M 0,-1 0,0',
                                                strokeOpacity: 1,
                                                scale: 4
                                            },
                                            offset: '0',
                                            repeat: '13px'
                                        }],
                                },
                                highlight: {
                                    strokeColor: ht_utility_1.Color.grey2,
                                    strokeOpacity: 1,
                                    strokeWeight: 5
                                },
                                fade: {
                                    strokeColor: ht_utility_1.Color.grey2,
                                    strokeOpacity: 0.2,
                                    strokeWeight: 2
                                }
                            };
                        }
                        ;
                    case "leaflet": {
                        return {
                            default: {
                                weight: 5,
                                color: ht_utility_1.Color.grey4,
                                opacity: 1,
                                dashArray: "7 10"
                            },
                            highlight: {
                                weight: 5,
                                color: ht_utility_1.Color.grey2,
                                opacity: 1
                            },
                            fade: {
                                weight: 2,
                                color: ht_utility_1.Color.grey2,
                                opacity: 0.2
                            }
                        };
                    }
                }
            },
        };
        this.name = "action expected polyline";
    }
    ActionsExpectedPolyline.prototype.isValidMapItems = function (data) {
        return !!this.getEncodedPath(data);
    };
    ActionsExpectedPolyline.prototype.getEncodedPath = function (data) {
        return data.expected_route;
    };
    ;
    ActionsExpectedPolyline.prototype.getPosition = function () {
        return { lat: 0, lng: 0 };
    };
    return ActionsExpectedPolyline;
}());
exports.ActionsExpectedPolyline = ActionsExpectedPolyline;
exports.ActionsExpectedPolylineTrace = data_observable_1.DataObservableMixin(trace_1.TraceMixin(extend_bounds_1.ExtendBoundsMixin(polyline_renderer_1.PolylinesMixin(marker_renderer_1.MarkersMixin(styles_1.StyleMixin(map_items_1.MapItemsMixin(ActionsExpectedPolyline)))))));
//# sourceMappingURL=actions-expected-polyline.js.map