"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var placeline_trace_1 = require("../compound-entities/placeline-trace");
var users_cluster_1 = require("../entities/users-cluster");
var ht_google_maps_wrapper_1 = require("ht-google-maps-wrapper");
var stops_heatmap_1 = require("../entities/stops-heatmap");
var actions_cluster_1 = require("../entities/actions-cluster");
var actions_heatmap_1 = require("../entities/actions-heatmap");
var map_instance_1 = require("../map-utils/map-instance");
var map_type_1 = require("./map-type");
var HtMapClass = /** @class */ (function () {
    // clusters = [];
    // map$ = new ReplaySubject();
    function HtMapClass(mapType, options) {
        if (mapType === void 0) { mapType = "leaflet"; }
        if (options === void 0) { options = {}; }
        this.mapType = mapType;
        // map: HtMap;
        // mapUtils: MapUtils;
        // userData$: Observable<IUserPlaceline | null>;
        this.mapInstance = new map_instance_1.MapInstance();
        this.leafletSetBoundsOptions = {
            animate: true,
            duration: 0.3
        };
        this.googleSetBoundsOptions = {};
        this.googleMapOptions = {
            center: { lat: 0, lng: 0 },
            zoom: 2,
            fullscreenControl: false,
            streetViewControl: false,
            styles: ht_google_maps_wrapper_1.LightColorMapStyle
        };
        this.leafletMapOptions = { center: [3.505, 0], zoom: 2 };
        this.mapItemsSet = [];
        map_type_1.mapTypeService.getInstance(mapType);
        if (options.mapKey) {
            // mapTypeService.getInstance().setKey(options.mapKey);
        }
        this.usersCluster = new users_cluster_1.UsersClusterTrace(this.mapInstance);
        this.actionsCluster = new actions_cluster_1.ActionsClusterTrace(this.mapInstance);
        this.usersHeatmap = new stops_heatmap_1.StopsHeatmapTrace(this.mapInstance);
        this.actionsHeatmap = new actions_heatmap_1.ActionsHeatmapTrace(this.mapInstance);
        this.placeline = new placeline_trace_1.PlacelineTrace({ mapInstance: this.mapInstance });
    }
    Object.defineProperty(HtMapClass.prototype, "segmentTrace", {
        get: function () {
            return this.placeline;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HtMapClass.prototype, "map$", {
        get: function () {
            return this.mapInstance.map$;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HtMapClass.prototype, "map", {
        get: function () {
            return this.mapInstance.map;
        },
        enumerable: true,
        configurable: true
    });
    HtMapClass.prototype.initMap = function (elem, options) {
        if (options === void 0) { options = {}; }
        var mapOptions = this.mapType == "leaflet"
            ? this.leafletMapOptions
            : this.googleMapOptions;
        var map = this.mapInstance.mapUtils.renderMap(elem, __assign({}, mapOptions, options));
        this.mapInstance.setMap(map);
        return map;
    };
    // setPlacelineData$(data$: Observable<IUserPlaceline | null>) {
    //   if (this.userDataSub) {
    //     this.userDataSub.unsubscribe();
    //   }
    //   this.initUserDataObserver(data$)
    // }
    //
    // private initUserDataObserver(data$: Observable<IUserPlaceline | null>) {
    //   let userData$ = data$.scan((acc, data) => {
    //     const oldId = acc.user ? acc.user.id : null;
    //     const currentId = data ? data.id : null;
    //     const isNew = currentId && oldId ? currentId !== oldId : true;
    //     return {user: data, isNew, oldId }
    //   }, {user: null, oldId: null, isNew: true});
    //
    //   let sub = userData$.subscribe((acc) => {
    //     const userData = acc.user;
    //     const isNew = acc.isNew;
    //     this.tracePlaceline(userData);
    //     if(isNew) this.resetBounds()
    //   });
    //   this.userDataSub = sub;
    // }
    HtMapClass.prototype.tracePlaceline = function (user) {
        this.placeline.trace(user);
    };
    HtMapClass.prototype.resetBounds = function (options) {
        this.mapInstance.resetBounds(options);
    };
    HtMapClass.prototype.getBoundsItem = function (items) {
        var _this = this;
        var bounds = this.mapInstance.mapUtils.extendItemBounds();
        return items.reduce(function (bounds, item) {
            return _this.getBounds(bounds, item);
        }, bounds);
    };
    HtMapClass.prototype.getBounds = function (bounds, item) {
        return item.extendBounds(bounds);
    };
    HtMapClass.prototype.setBounds = function (bounds, options) {
        options =
            options || this.mapType == "leaflet"
                ? this.leafletSetBoundsOptions
                : this.googleSetBoundsOptions;
        this.mapInstance.mapUtils.setBounds(this.map, bounds, options);
    };
    HtMapClass.prototype.inValidateSize = function () {
        this.mapInstance.inValidateSize();
    };
    HtMapClass.prototype.addEntities = function (entities) {
        this.mapInstance.addToItemsSet(entities);
    };
    HtMapClass.prototype.clear = function () {
        this.mapInstance.setMap(null);
    };
    return HtMapClass;
}());
exports.HtMapClass = HtMapClass;
//# sourceMappingURL=map-class.js.map