import { BehaviorSubject, Observable, Subscription } from "rxjs";
import { HtPosition, IPathBearingTime } from "ht-models";
import { Constructor, Entities, Entity } from "../interfaces";
import { MapInstance } from "../map-utils/map-instance";
import { HtCustomEvent, IEventSub } from "ht-utility";
import { IMarkersArray, SetDataConfig } from "./data-observable";
export interface IMapItemsMixinBase {
    mapInstance: MapInstance;
    trackBy?(datum: any): string;
    trackAnimationBy?(datum: any): string;
}
export declare function MapItemsMixin<TBase extends Constructor<IMapItemsMixinBase>>(Base: TBase): {
    new (...args: any[]): {
        entities: Entities<any>;
        event: HtCustomEvent;
        htShow(item: any): string;
        getEntity(id?: string): Entity<any>;
        trackBy(datum: any): any;
        trackAnimationBy(datum: any): any;
        onEntityUpdate(id: any): {
            subscribe: (cb: any) => IEventSub;
        };
        removeItem(item: any): void;
        removeAll(entities: any): void;
        clear(): void;
        clearItem({item, data}: {
            item: any;
            data: any;
        }): void;
        removeData(data: any): void;
        mapInstance: MapInstance;
    };
} & TBase;
export interface IMapItemsBase {
    getItem: (data) => any;
    setStyle: (style) => void;
    update: () => void;
    mapInstance: MapInstance;
    trackBy(datum: any): string;
    getBounds: (data) => any;
}
export declare function MapItemTrace<TBase extends Constructor<IMapItemsBase>>(Base: TBase): {
    new (...args: any[]): {
        dataSub: Subscription;
        dataPageSource$: Observable<{
            count: number;
            next: string;
            previous: string;
            results: any[];
        }>;
        dataArraySource$: Observable<any[]>;
        data$: Observable<IMarkersArray>;
        traceData$: BehaviorSubject<any[]>;
        _trace(data?: any[], map?: any): void;
        _procData$(): (source$: Observable<{
            count: number;
            next: string;
            previous: string;
            results: any[];
        }>) => Observable<IMarkersArray>;
        getMarkersArray(array: any[], isNew?: boolean): {
            valid: any[];
            invalid: any[];
            isNew: boolean;
        };
        setPageData$(data$: Observable<{
            count: number;
            next: string;
            previous: string;
            results: any[];
        }>, config?: SetDataConfig): void;
        setData$(data$: Observable<any[]>, config?: SetDataConfig): void;
        _initDataObserver(): void;
        emptyRenderData(): IMarkersArray;
        clear(): void;
        trace: (data: any, map?: any) => any;
        isValidMapItems?: (data: any) => boolean;
        getPosition?: (data: any) => HtPosition;
        mapInstance: MapInstance;
    };
} & {
    new (...args: any[]): {
        trace(data: any[], map?: any): boolean;
        traceItem(datum: any): void;
        addEvents(item: any, id: any): void;
        bustOldItem(): void;
        name?: string;
        getItem: (data: any) => any;
        onMouseLeave?: (trace: any) => void;
        onClick?: (trace: any) => void;
        onMouseEnter?: (trace: any) => void;
        setStyle: (entity: Entity<any>) => void;
        update: (entity: any, pathBearing?: IPathBearingTime) => void;
        traceEffect?: () => void;
        removeAll: (entities: any) => void;
        getBounds: (item: any, bounds?: any) => any;
        removeItem: (item: any) => void;
        removeData: (data: any) => void;
        clearItem: (entity: Entity<any>) => void;
        toNotSetMap?: boolean;
        toNotTraceItem?: boolean;
        trackBy(datum: any): string;
        entities: Entities<any>;
        mapInstance: MapInstance;
    };
} & {
    new (...args: any[]): {
        entities: Entities<any>;
        event: HtCustomEvent;
        htShow(item: any): string;
        getEntity(id?: string): Entity<any>;
        trackBy(datum: any): any;
        trackAnimationBy(datum: any): any;
        onEntityUpdate(id: any): {
            subscribe: (cb: any) => IEventSub;
        };
        removeItem(item: any): void;
        removeAll(entities: any): void;
        clear(): void;
        clearItem({item, data}: {
            item: any;
            data: any;
        }): void;
        removeData(data: any): void;
        mapInstance: MapInstance;
    };
} & TBase;
