import { Constructor } from "../interfaces";
import { Subscription ,  Observable ,  combineLatest , of} from "rxjs";
// import { GlobalMap } from "../global/map-service";
import { distinctUntilChanged ,  filter ,  map ,  scan , skip, skipUntil} from "rxjs/operators";
import * as _ from "underscore";
import { dataWithSelectedId$ } from "ht-data";
import { HtPosition} from "ht-models";
import { orCombine } from "ht-data";
import {MapInstance} from "../map-utils/map-instance";
import {take} from "rxjs/operator/take";
import {IMarkersArray} from "./data-observable";

export interface ICompoundsDataObservableBase {
  trace: (data, map?) => any;
  isValidMapItems?: (data) => boolean;
  mapInstance: MapInstance,
  toResetMap?: (newData, oldData) => boolean
  // getPosition: (data) => HtPosition;
}

export function CompoundDataObservableMixin<
  TBase extends Constructor<ICompoundsDataObservableBase>
>(Base: TBase) {
  return class extends Base {
    dataSub: Subscription;
    dataSource$: Observable<object | null>;
    data$: Observable<object>;
    compoundSetDataConfig: CompoundSetDataConfig;

    constructor(...args: any[]) {
      super(...args);
      this.mapInstance.addToItemsSet(this)
    }

    _procData$() {
      return (source$: Observable<any | null>) => {
        return source$.pipe(
          map(markers => {
            return markers.reduce(
              (acc, item) => {
                const isValid = true;
                if (isValid) {
                  acc.valid.push(item);
                } else {
                  acc.invalid.push(item);
                }
                return acc;
              },
              { valid: [], invalid: [] }
            );
            // return markers
          })
        );
      };
    }

    setCompoundData$(
      data$: Observable<object | null>,
      config: CompoundSetDataConfig = {}
    ) {
      this.compoundSetDataConfig = config;
      if (this.dataSub) {
        this.dataSub.unsubscribe();
      }
      const hide$ = config.hide$;
      const filter$ = config.filter$;
      let dataSource$ = hide$
        ? combineLatest(
            data$,
            hide$.pipe(distinctUntilChanged()),
            (data, hide) => (!!hide ? null : data)
          )
        : data$;
      if (config.roots && config.highlighted$) {
        dataSource$ = dataWithSelectedId$(dataSource$, config.highlighted$, config.roots, 'highlightedSegment');
      }
      if (config.roots && filter$)
        dataSource$ = dataWithSelectedId$(dataSource$, filter$, config.roots, 'selectedSegment');

      this.dataSource$ = dataSource$;

      this.data$ = this.dataSource$;

      this._initDataObserver();
    }

    _initData$() {

      let mapData$ = this.data$;
      let render$ = combineLatest(
        mapData$,
        this.mapInstance.map$
      ).pipe(
        map(([mapData, map]) => {
          return map ? mapData : null;
        })
      );
      // function isNewId (newItem, old) {
      //   if(!old && newItem) return true;
      //   if(newItem && old) return  newItem.id !== old.id
      // }
      // function isNewList(newList, old) {
      //   if(!old && newList) return true;
      //   if(newList && old) return !newList.next && newList.count
      // }
      // let sub = render$.subscribe(({ valid, invalid, isNew }) => {
      //   this.trace(valid);
      //   if (isNew) this.mapInstance.resetBounds();
      // });
      // this.dataSub = sub;

      // here
      let userData$ = render$.pipe(
        scan(
          (acc: { user: any; oldUser: any }, data: object) => {
            const oldUser = acc.user;
            return { user: data, oldUser };
          },
          { user: null, oldUser: null }
        )
      );
      return userData$;
    }

    _initDataObserver() {
      let userData$ = this._initData$();
      const toResetMap = this.toResetMap;
      function isNewItem(newItem, old) {
        if (toResetMap) return toResetMap(newItem, old);
        if (newItem && old) {
          return newItem.id !== old.id || (newItem.timeline_date !== old.timeline_date)
        } else if (!old && newItem) {
          return true
        } else {
          return false
        };
      }

      let newPlaceline$ = userData$.pipe(
        map((acc: { user: any; oldUser: any }) => {
          const userData = acc.user;
          this.trace(userData);
          const isNew = isNewItem(acc.user, acc.oldUser);
          return isNew;
          // if(isNew) GlobalMap.resetBounds()
        })
      );

      const resetMap$ = this.compoundSetDataConfig.resetMap$ ?
        this.compoundSetDataConfig.resetMap$.pipe(
        map(data => true)
      ) : of(true);

      let sub = orCombine(
        newPlaceline$.pipe(filter(data => !!data)),
        resetMap$
      ).subscribe(toReset => {
        if (toReset) this.mapInstance.resetBounds();
      });
      this.dataSub = sub;
    }
  };
}

export interface CompoundSetDataConfig {
  hide$?: Observable<any>;
  filter$?: Observable<string | null>;
  roots?: string[];
  resetMap$?: Observable<any>;
  highlighted$?: Observable<string | null>
}
