import { Constructor } from "../interfaces";
import { Subscription ,  Observable ,  combineLatest , BehaviorSubject} from "rxjs";
// import { GlobalMap } from "../global/map-service";
import { distinctUntilChanged ,  filter ,  map ,  pluck ,  scan , tap} from "rxjs/operators";
import * as _ from "underscore";
import { HtPosition } from "ht-models";
import { AllData, Page } from "ht-models";
import {MapInstance} from "../map-utils/map-instance";

export interface IMarkersArray {
  valid: any[];
  invalid: any[];
  isNew: boolean;
}

export interface IDataObservableBase {
  trace: (data, map?) => any;
  isValidMapItems?: (data) => boolean;
  getPosition: (data) => HtPosition;
  mapInstance: MapInstance
}
export function DataObservableMixin<
  TBase extends Constructor<IDataObservableBase>>(Base: TBase) {
  return class extends Base {
    dataSub: Subscription;
    dataPageSource$: Observable<Page<any>>;
    dataArraySource$: Observable<any[]>;
    data$: Observable<IMarkersArray>;
    traceData$: BehaviorSubject<any[]>;
    constructor(...args: any[]) {
      super(...args);
    }

    _trace(data: any[] = [], map?) {
      if (!this.traceData$) {
        this.traceData$ = new BehaviorSubject<any[]>(data)
      } else {
        this.traceData$.next(data)
      }
      this.setData$(this.traceData$)
    }

    _procData$() {
      return (source$: Observable<Page<any>>) => {
        return source$.pipe(
          // filter(data => !!data),
          map(pageData => {
            if (pageData) {
              let isNew = pageData && pageData.count && !pageData.previous;
              let results = pageData ? pageData.results : [];
              return this.getMarkersArray(results, isNew);
            } else {
              return this.emptyRenderData()
            }

          })
        );
      };
    }

    getMarkersArray(markersArray: any[] = [], isNew: boolean = false) {
      return markersArray.reduce(
        (acc, item) => {
          const isValid = this.isValidMapItems
            ? this.isValidMapItems(item)
            : !!this.getPosition(item);
          if (isValid) {
            acc.valid.push(item);
          } else {
            acc.invalid.push(item);
          }
          return acc;
        },
        { valid: [], invalid: [], isNew }
      );
    }

    setPageData$(
      data$: Observable<Page<any> | null>,
      config: SetDataConfig = {}
    ) {
      if (this.dataSub) {
        this.dataSub.unsubscribe();
      }
      const hide$ = config.hide$;
      this.dataPageSource$ = hide$
        ? combineLatest(
            data$,
            hide$.pipe(distinctUntilChanged()),
            (data, hide) =>
              !!hide ? { results: [], count: 0, next: "", previous: "" } : data
          )
        : data$;
      this.data$ = this.dataPageSource$.pipe(this._procData$());
      this._initDataObserver();
    }

    setData$(data$: Observable<any[]>, config: SetDataConfig = {}) {
      if (this.dataSub) {
        this.dataSub.unsubscribe();
      }
      interface ScanData {
        new: any[],
        old: any[]
      };
      const hide$ = config.hide$;
      this.dataArraySource$ = hide$
        ? combineLatest(
            data$,
            hide$.pipe(distinctUntilChanged()),
            (data, hide) => (!!hide ? [] : data)
          )
        : data$;
      this.data$ = this.dataArraySource$.pipe(
        scan((acc: ScanData, data: any[]) => {
          return {new: data, old: acc.new}
        }, {old: null, new: null}),
        map((dataArray: ScanData) => {
          return this.getMarkersArray(dataArray.new || [], !dataArray.old);
        })
      );
      this._initDataObserver();
    }

    // _initData$() {
    //   let userData$ = this.dataPageSource$.pipe(
    //     filter(data => !!GlobalMap.map),
    //     pluck('valid'),
    //     scan((acc: {user: any, oldUser: any}, data: object) => {
    //       const oldUser = acc.user;
    //       return {user: data, oldUser }
    //     }, {user: null, oldUser: null})
    //   );
    //   return userData$;
    // };

    _initDataObserver() {
      let mapData$ = this.data$;
      let render$ = combineLatest(
        mapData$,
        this.mapInstance.map$
      ).pipe(
        map(([mapData, map]) => {
          return map ? mapData : this.emptyRenderData();
        })
      );
      // function isNewId (newItem, old) {
      //   if(!old && newItem) return true;
      //   if(newItem && old) return  newItem.id !== old.id
      // }
      // function isNewList(newList, old) {
      //   if(!old && newList) return true;
      //   if(newList && old) return !newList.next && newList.count
      // }
      let sub = render$.subscribe(({ valid, invalid, isNew }) => {
        this.trace(valid);
        if (isNew) this.mapInstance.resetBounds();
      });
      this.dataSub = sub;
    }

    emptyRenderData (): IMarkersArray {
      return {valid: [], invalid: [], isNew: false}
    }

    clear() {
      this.dataSub.unsubscribe();
    }
  };
}

export interface SetDataConfig {
  hide$?: Observable<any>;
}
